
from CrawlSpider.spiders.BaseNewsSpider import BaseNewsSpider
from CrawlSpider.ETL.BaseETL import baseETL


class NewsSpider(BaseNewsSpider):
    async def saveTo(self, resultItem:dict):
        raise NotImplemented('`saveTo` for NewSpider is not implemented')

    def keyMap(self):
        ks = "url,title,publish_time,author,content,source,originalContent".split(',')
        kMap = self.getKMap()
        return ks, kMap

    async def transfer(self,fieldDic: dict, **kwargs):
        new_content = fieldDic.get('content') or fieldDic.get('body')
        data = {
            'new_content': new_content,
            'url': fieldDic.get('url')
        }
        result = await baseETL.bodyTransfer(data)
        content = result.get('new_content')
        originalContent = content
        if self.isDelTagsExceptImg():
            content = await baseETL.delTagsExceptImg(content)
        fieldDic.update({
            'content': content,
            'originalContent': originalContent
        })
        return await super().transfer(fieldDic, **kwargs)


    def isDelTagsExceptImg(self):
        return True



















# -*- coding: utf-8 -*-
# __author__ : Ricky
# __createTime__ : 2022/7/8 1:49
# __fileName__ : CrawlSpider splash.py
# __devIDE__ : PyCharm


import asyncio
from CrawlSpider.Utils.SpiderRequest import spiderRequest

async def get(url):
    res = await spiderRequest.get(
        url,
        isSplash=True
    )
    print(res['content'].decode())

if __name__ == '__main__':
    # asyncio.run(get("http://httpbin.org/get"))
    asyncio.run(get("https://haokan.baidu.com/tab/shenghuo_new"))











































































































# -*- coding: utf-8 -*-
# __author__ : Ricky
# __createTime__ : 2019/11/13 17:26
# __fileName__ : Python_Study SQLibs.py
# __devIDE__ : PyCharm
import pymysql
from dbutils.pooled_db import PooledDB
from contextlib import contextmanager
import traceback
from math import floor

class SQLib:
    def __init__(self, connect_params):
        self.connect_params = connect_params
        self.instanced()

    def instanced(self):
        # , ping=2
        self.poolDB = PooledDB(pymysql, mincached=10, maxcached=20, maxconnections=50, blocking=True, **self.connect_params)

    def get_conn(self):
        if self.poolDB is None:
            self.instanced()
        while 1:
            try:
                return self.poolDB.connection()
            except Exception:
                print(traceback)

    @contextmanager
    def selectAll(self, sql):
        conn_python = self.get_conn()
        cursor_python = conn_python.cursor(pymysql.cursors.DictCursor)
        try:
            cursor_python.execute(sql)
            result = cursor_python.fetchall()

            yield result
        except Exception as e:
            print(traceback.format_exc())
            yield None
        finally:
            cursor_python.close()
            conn_python.close()

    @contextmanager
    def getCount(self, sql):
        if ('count' not in sql.lower()):
            yield 'NoCount'
        else:
            conn_python = self.get_conn()
            cursor_python = conn_python.cursor(pymysql.cursors.DictCursor)
            # sql = 'INSERT INTO {table}({keys}) VALUES ({values})'.format(table=table, keys=keys, values=values)
            try:
                cursor_python.execute(sql)
                result = cursor_python.fetchall()
                # print(result[0])
                yield result
            except Exception as e:
                print(traceback.format_exc())
                yield None
            finally:
                cursor_python.close()
                conn_python.close()

    @contextmanager
    def insertMany(self, sql, params):
        conn_python = self.get_conn()
        cursor_python = conn_python.cursor()
        yield sql
        try:
            cursor_python.executemany(sql, params)
            conn_python.commit()
        except Exception as e:
            for param in params:
                print(param)
            conn_python = self.get_conn()
            cursor_python = conn_python.cursor()
            print(traceback.format_exc())
            print(sql)
            write_flag = 0
            length = len(params)

            if (length > 1000):
                count = floor(length / 1000) + 1
                try:
                    for i in range(count):
                        res = params[i * 1000:(i + 1) * 1000]
                        cursor_python.executemany(sql, res)
                        conn_python.commit()
                except Exception as e:
                    print(e)
            else:
                for param in params:
                    try:
                        cursor_python.execute(sql, param)
                        conn_python.commit()
                    except Exception as e:
                        write_flag += 1
                        print(e)
                        print(f"当前报错param:{param}")
                        conn_python.rollback()
        finally:
            cursor_python.close()
            conn_python.close()

    @contextmanager
    def insertOneByOne(self, sql, params):
        conn_python = self.get_conn()
        cursor_python = conn_python.cursor()
        yield sql
        for param in params:
            try:
                # print(params[0])
                cursor_python.execute(sql, param)
                conn_python.commit()
            except Exception as e:
                print(e)
                print(param)
                print(sql)
                print(traceback.format_exc())
                conn_python.rollback()
        else:
            cursor_python.close()
            conn_python.close()

    @contextmanager
    def delete(self, sql, params=None):
        conn_python = self.get_conn()
        cursor_python = conn_python.cursor()
        yield sql
        try:
            if (params):
                cursor_python.execute(sql, params)
            else:
                cursor_python.execute(sql)
            conn_python.commit()
        except Exception as e:
            print(e)
            conn_python.rollback()
        finally:
            cursor_python.close()
            conn_python.close()

    @contextmanager
    def update(self, sql, params=None):
        conn_python = self.get_conn()
        cursor_python = conn_python.cursor()
        yield sql
        try:
            if (params):
                cursor_python.execute(sql, params)
            else:
                cursor_python.execute(sql)
            conn_python.commit()
        except Exception as e:
            print(e)
            conn_python.rollback()
        finally:
            cursor_python.close()
            conn_python.close()





# -*- coding: utf-8 -*-
# __author__ : Ricky
# __createTime__ : 2022/7/11 16:03
# __fileName__ : CrawlSpider DateUtils.py
# __devIDE__ : PyCharm

from dateutil.relativedelta import relativedelta
from CrawlSpider.date_infer import infer
import time
import calendar
import datetime

class DateUtils:
    sep = ''

    @classmethod
    def strToDateTime(cls, dateTime: str, form=None):
        try:
            form = form or infer([dateTime])
            ret = datetime.datetime.strptime(dateTime, form or '%Y-%m-%d %H:%M:%S')
        except Exception as e:
            ret = cls.strToDateTime(str(datetime.datetime.today() - datetime.timedelta(days=1)).split('.')[0])
        return ret

    @staticmethod
    def dateTimeToStr(dateTime: datetime.datetime, form=None):
        return dateTime.strftime(form or '%Y-%m-%d %H:%M:%S')

    @classmethod
    def dateIsOk(cls, currDate: datetime.datetime, period=None, days=1):
        startDate, endDate = cls.getStartEndDate(period, days)
        return startDate <= currDate.date() < endDate

    @staticmethod
    def getStartEndDate(period=None, days=1):
        if period:
            startDate = datetime.datetime.strptime(period[0], '%Y%m%d').date()
            endDate = datetime.datetime.strptime(period[-1], '%Y%m%d').date()
        else:
            today = datetime.datetime.today().date()
            startDate = today - datetime.timedelta(days=days)
            endDate = today
        return startDate, endDate
    
    @classmethod
    def get_last_day(cls, day=None, number=1):
        # 获取前几天
        d = (day and datetime.datetime.strptime(day,
                                                f"%Y{cls.sep}%m{cls.sep}%d") or datetime.datetime.now().date()) - relativedelta(
            days=number)
        return d.strftime(f"%Y{cls.sep}%m{cls.sep}%d")

    @classmethod
    def dayPlusN(cls, currDay, delta=1, dateFormat='%Y%m%d'):
        # 获取后几天
        startTime = datetime.datetime.strptime(str(currDay), dateFormat)
        ttd = (startTime + datetime.timedelta(days=delta)).strftime(dateFormat)
        return ttd
    
    @classmethod
    def get_num_in_month(cls, day=None):
        # 获取当前月

        d = (day and datetime.datetime.strptime(day,
                                                f"%Y{cls.sep}%m{cls.sep}%d") or datetime.datetime.now()) - datetime.datetime.strptime(
            cls.get_cur_month_start(), f"%Y{cls.sep}%m{cls.sep}%d")
        return d.days

    @classmethod
    def get_cur_month(cls):
        # 获取当前月
        m = datetime.datetime.now().strftime(f"%Y{cls.sep}%m")
        return "{}".format(m)
    
    @classmethod
    def get_last_month(cls, number=1):
        # 获取前几个月
        month_date = datetime.datetime.now().date() - relativedelta(months=number)
        m = month_date.strftime(f"%Y{cls.sep}%m")
        return "{}".format(m)
    
    @classmethod
    def get_next_month(cls, number=1):
        # 获取后几个月
        month_date = datetime.datetime.now().date() + relativedelta(months=number)
        m = month_date.strftime(f"%Y{cls.sep}%m")
        return "{}".format(m)
    
    @classmethod
    def get_cur_month_start(cls, day=None):
        # 获取当前月的第一天
        month_str = (day and datetime.datetime.strptime(day,
                                                        f"%Y{cls.sep}%m{cls.sep}%d") or datetime.datetime.now()).strftime(
            f'%Y{cls.sep}%m')
        return f'{month_str}{cls.sep}01'
    
    @classmethod
    def get_cur_month_end(cls):
        # 获取当前月的最后一天
        '''
        param: month_str 月份，2021-04
        '''
        # return: 格式 %Y-%m-%d

        month_str = datetime.datetime.now().strftime('%Y-%m')
        year, month = int(month_str.split('-')[0]), int(month_str.split('-')[1])
        end = calendar.monthrange(year, month)[1]
        return f'{year}{cls.sep}{month}{cls.sep}{end}'
    
    @classmethod
    def get_last_month_start(cls, month_str=None):
        # 获取上一个月的第一天
        '''
        param: month_str 月份，2021-04
        '''
        # return: 格式 %Y-%m-%d
        if not month_str:
            month_str = datetime.datetime.now().strftime('%Y-%m')
        year, month = int(month_str.split('-')[0]), int(month_str.split('-')[1])
        if month == 1:
            year -= 1
            month = 12
        else:
            month -= 1
        return f'{year}{cls.sep}{month}-01'
    
    @classmethod
    def get_next_month_start(cls, month_str=None):
        # 获取下一个月的第一天
        '''
        param: month_str 月份，2021-04
        '''
        # return: 格式 %Y-%m-%d
        if not month_str:
            month_str = datetime.datetime.now().strftime('%Y-%m')
        year, month = int(month_str.split('-')[0]), int(month_str.split('-')[1])
        if month == 12:
            year += 1
            month = 1
        else:
            month += 1
        return f'{year}{cls.sep}{month}{cls.sep}01'
    
    @classmethod
    def get_last_month_end(cls, month_str=None):
        # 获取上一个月的最后一天
        '''
        param: month_str 月份，2021-04
        '''
        # return: 格式 %Y-%m-%d
        if not month_str:
            month_str = datetime.datetime.now().strftime('%Y-%m')
        year, month = int(month_str.split('-')[0]), int(month_str.split('-')[1])
        if month == 1:
            year -= 1
            month = 12
        else:
            month -= 1
        end = calendar.monthrange(year, month)[1]
        return f'{year}{cls.sep}{month}{cls.sep}{end}'
    
    @classmethod
    def get_next_month_end(cls, month_str=None):
        # 获取下一个月的最后一天
        '''
        param: month_str 月份，2021-04
        '''
        # return: 格式 %Y-%m-%d
        if not month_str:
            month_str = datetime.datetime.now().strftime('%Y-%m')
        year, month = int(month_str.split('-')[0]), int(month_str.split('-')[1])
        if month == 12:
            year += 1
            month = 1
        else:
            month += 1
        end = calendar.monthrange(year, month)[1]
        return f'{year}{cls.sep}{month}{cls.sep}{end}'

    @classmethod
    # 工具方法之获取两个时间戳之间的间隔(这里统一精度为13位)
    def getTimeDelta(cls, startTime, endTime):  # 精度到毫秒(13位时间戳)
        delta = endTime - startTime
        symbol_flag = False
        if delta < 0:
            symbol_flag = True
            delta = float(str(delta).split('-')[-1].strip())
        timedelta = str(datetime.timedelta(0.0, 0.0, 0.0, delta))
        timedelta = timedelta.replace('day', '')
        timedelta = timedelta.replace('.', ',')
        result = timedelta.split(',')
        result[-1] = str(result[-1])[0]
        if symbol_flag:
            result.append('-')
        else:
            result.append('+')
        return result

    @classmethod
    def getTimeStamp(cls, dateTime=None, DateFormat='%Y-%m-%d'):
        return int(round(time.time())) if not dateTime else int(time.mktime(time.strptime(dateTime, DateFormat)))

    @classmethod
    def timestamp2dateTime(cls, timestamp):
        formatTime = '%Y-%m-%d %H:%M:%S'
        if (len(str(timestamp)) == 13):
            timestamp = timestamp / 1000
        return time.strftime(formatTime, time.localtime(timestamp))








import traceback
import psycopg
from dbutils.persistent_db import PersistentDB
from dbutils.pooled_db import PooledDB


class PgDb:
    """A lightweight wrapper around psycopg for easy to use
    maxconnections=6,  # 连接池允许的最大连接数，0和None表示不限制连接数
    mincached=2,  # 初始化时，链接池中至少创建的空闲的链接，0表示不创建
    maxcached=5,  # 链接池中最多闲置的链接，0和None不限制
    maxshared=3,  # 链接池中最多共享的链接数量，0和None表示全部共享。PS: 无用，因为pymysql和MySQLdb等模块的 threadsafety都为1，所有值无论设置为多少，_maxcached永远为0，所以永远是所有链接都共享。
    blocking=True,  # 连接
    """
    def __init__(self,
                 creator=psycopg,
                 maxusage=None,
                 dbname=None,
                 user=None,
                 host='127.0.0.1',
                 password=None,
                 isShare=False,
                 setsession=None,
                 failures=None,
                 ping=1,
                 closeable=False,
                 threadlocal=None,
                 maxconnections=20,
                 mincached=5,
                 maxcached=10,
                 maxshared=3,
                 *args,
                 **kwargs):
        """
        referer psycopg PersistentDB init
        :param creator:
        :param maxusage:threadlocal
        :param dbname:
        :param user:
        :param host:
        :param password:
        :param setsession:
        :param failures:
        :param ping:
        :param closeable:
        :param threadlocal:
        :param args:
        :param kwargs:
        """
        self.creator = creator
        self.args = args
        if not isShare:
            kwargs.update(dict(
                maxusage=maxusage,
                dbname=dbname,
                user=user,
                host=host,
                password=password,
                setsession=setsession,
                failures=failures,
                ping=ping,
                closeable=closeable,
                threadlocal=threadlocal,
            ))
        else:
            kwargs.update(dict(
                maxusage=maxusage,
                dbname=dbname,
                user=user,
                host=host,
                password=password,
                setsession=setsession,
                failures=failures,
                ping=ping,
                maxconnections=maxconnections,
                mincached=mincached,
                maxcached=maxcached,
                maxshared=maxshared,
            ))
        self.kwargs = kwargs
        self.pool = PersistentDB(creator, *args, **kwargs) if not isShare else PooledDB(creator, *args, **kwargs)

    @staticmethod
    def getDict(description, rows):
        """
        use description and fetchResult to transfer list to dict
        :param description:
        :param rows:
        :return:
        """
        res = {}
        if rows:
            list_header = [row[0] for row in description]
            list_result = [[item for item in row] for row in rows]
            res = [dict(zip(list_header, row)) for row in list_result]
        return res

    def connection(self):
        """
        get a valid connection
        :return:
        """
        if not self.pool:
            self.pool = PersistentDB(self.creator, *self.args, **self.kwargs)
        return self.pool.connection()

    def fetchall(self, query):
        """Returns a List[dict] for the given query and parameters."""
        conn = self.connection()
        with conn.cursor() as cur:
            try:
                cur.execute(query)
                res = self.getDict(cur.description, cur.fetchall())
            except Exception as e:
                conn._ping_check()
                cur.execute(query)
                res = self.getDict(cur.description, cur.fetchall())
            return res

    def fetchone(self, query):
        """Returns a row dict for the given query and parameters."""
        conn = self.connection()
        with conn.cursor() as cur:
            try:
                cur.execute(query)
                item = cur.fetchone()

                rows = item and [item] or []
                res = self.getDict(cur.description, rows) or []
                if res:
                    res = res[0]
            except Exception as e:
                conn._ping_check()
                cur.execute(query)
                item = cur.fetchone()
                rows = item and [item] or []
                res = self.getDict(cur.description, rows) or []
                if res:
                    res = res[0]
            finally:
                cur.close()
                conn.close()
            return res


    def fetchmany(self, query, size=1000):
        """Returns a row dict for the given query and parameters."""
        conn = self.connection()
        res = []
        with conn.cursor() as cur:
            try:
                cur.execute(query)

                while 1:
                    itemList = cur.fetchmany(size=size)
                    res = self.getDict(cur.description, itemList)
                    if len(res) < size:
                        break

                    yield res

            except Exception as e:
                conn._ping_check()
                self.fetchmany(query=query, size=size)
            finally:
                cur.close()
                conn.close()
            yield res

    def execute(self, query, *parameters, **kwparameters):
        """Executes the given query"""
        conn = self.connection()
        with conn.cursor() as cur:
            try:
                cur.execute(query, kwparameters or parameters)
                conn.commit()
            except Exception as e:
                conn.rollback()
                return e.args[-1]
            finally:
                cur.close()
                conn.close()



    def executeMany(self, query, *parameters, **kwparameters):
        """Executes the given query, batch execute"""
        conn = self.connection()
        with conn.cursor() as cur:
            try:
                cur.executemany(query, kwparameters or parameters)
                conn.commit()
            except Exception as e:
                conn.rollback()
                return e.args[-1]
            finally:
                cur.close()
                conn.close()


    def table_has(self, table_name, conditions):
        """
        judge record that  is or not in table
        :param table_name:
        :param field:
        :param value:
        :return:
        """
        condition, _ = self.flat(conditions, condition=True, sep=' and ')
        query = f'SELECT * FROM {table_name} WHERE {condition}'
        res = self.fetchone(query)
        if res and len(res):
            return True
        return False

    def table_insert(self, table_name, item, ignore_duplicated=True):
        """
        insert data(dict) to table
        :param table_name:
        :param item:
        :param ignore_duplicated:
        :return:
        """
        fields = list(item.keys())
        values = list(item.values())
        fieldstr = ','.join(fields)
        valstr = ','.join(['%s'] * len(item))
        sql = 'INSERT INTO %s (%s) VALUES(%s)' % (table_name, fieldstr, valstr)
        try:
            last_id = self.execute(sql, *values)
            return last_id
        except Exception as e:
            if ignore_duplicated and e.args[0] == 1062:
                # just skip duplicated item
                return 0
            traceback.print_exc()
            print('sql:', sql)

            for i in range(len(fields)):
                vs = str(values[i])
                if len(vs) > 300:
                    print(fields[i], ' : ', len(vs), type(values[i]))
                else:
                    print(fields[i], ' : ', vs, type(values[i]))
            return e

    def table_insertMany(self, table_name, itemList):
        """
        insert many data==> List(dict) to table
        :param table_name:
        :param itemList:
        :return:
        """
        if not itemList:
            raise ValueError("itemList can't None")
        item:dict = itemList[0]
        ks = item.keys()
        fieldstr = ','.join(ks)
        valstr = ','.join(['%s'] * len(ks))
        sql = 'INSERT INTO %s (%s) VALUES(%s)' % (table_name, fieldstr, valstr)
        try:
            last_id = self.executeMany(sql, *[tuple(item.values()) for item in itemList])
            return last_id
        except Exception as e:
            traceback.print_exc()
            print('sql:', sql)
            return e

    @staticmethod
    def flat(updates, condition=False, sep=','):
        upsets = []
        values = []
        for k, v in updates.items():
            s = '%s=%%s' % k
            if condition:
                s = "%s='%s'" % (k, v)

                if isinstance(v, tuple):
                    s = f"{k} in {v}"
                elif 'null' in v.lower():
                    s = f"{k} is {v}"

            upsets.append(s)
            values.append(v)
        upsets = sep.join(upsets)

        return upsets, values

    def table_update(self, table_name, updates,
                     conditions):
        """
        according to condtions(dict) to update some field (updates) ==> dict
        :param table_name:
        :param updates:
        :param conditions:
        :return:
        """

        upsets, values = self.flat(updates)
        condition, _ = self.flat(conditions, condition=True, sep=' and ')
        sql = 'UPDATE %s SET %s WHERE %s' % (
            table_name,
            upsets,
            condition
        )
        self.execute(sql, *(values))


if __name__ == '__main__':
    pg_db = PgDb(dbname='spider', user="postgres", host='127.0.0.1', isShare=True)
    # example fetchone
    pg_db.fetchone('select * from test')
    # example fetchmany
    for itemList in pg_db.fetchmany('select * from test', size=3):
        print(itemList)



    # example insert one
    # res = pg_db.table_insert('plat_info', {
    #     'platform': 'wanfang',
    #     'view_desc': 'search',
    #     'view_url': 'https://s.wanfangdata.com.cn/thesis?q=专业%3A"一般力学与力学基础"'
    # })
    # print(res)
    # example insert many
    # res = pg_db.table_insertMany('plat_info', [
    #     {
    #         'platform': 'publons',
    #         'view_desc': 'detail',
    #         'view_url': 'https://publons.com/researcher/A-1001-2010/'
    #     },
    #     {
    #         'platform': 'publons',
    #         'view_desc': 'stats',
    #         'view_url': 'https://publons.com/api/stats/individual/2878410/'
    #     },
    #     {
    #         'platform': 'publons',
    #         'view_desc': 'profile_summary',
    #         'view_url': 'https://publons.com/api/profile/summary/2878410/'
    #     },
    #     {
    #         'platform': 'publons',
    #         'view_desc': 'profile_publication',
    #         'view_url': 'https://publons.com/api/profile/publication/2878410/?order_by=citations&page=1&per_page=5'
    #     }
    # ])
    # print(res)

    # example update
    # res = pg_db.table_update('wf_search_result', {
    #     'kw': '作者单位:电子科技大学'
    # },{
    #     'kw': '作者单位:("电子科技大学")'
    # })
    # print(res)

    # judge record is or not in table
    # print(pg_db.table_has('plat_info', {
    #     'platform': 'wanfang',
    #     'view_desc': 'detail'
    # }))






import os

from qcloud_cos import CosConfig
from qcloud_cos import CosS3Client
from uuid import uuid4
import sys
import logging
from CrawlSpider.conf.settings import logger

import imghdr


logging.basicConfig(level=logger['level'], stream=sys.stdout)




class CosOp:
    IMG_MAX_SIZE = 5 * 1024 * 1024
    IMG_THUMBNAIL_SIZE = 200 * 1024

    def __init__(self, cosConf):
        secret_id = cosConf['secretId']  # 替换为用户的 secretId(登录访问管理控制台获取)
        secret_key = cosConf['secretKey']  # 替换为用户的 secretKey(登录访问管理控制台获取)
        region = cosConf['regionId']  # 替换为用户的 Region
        config = CosConfig(Region=region, SecretId=secret_id, SecretKey=secret_key)
        # 2. 获取客户端对象
        self.client:CosS3Client = CosS3Client(config)
        self.cosConf = cosConf

    def getInfo(self, fileName:str, Content:bytes, rule=None):

        flag, msg = self.isImage(top32Bit=Content[:32])
        suffix = fileName.split('.')[-1] if not flag else msg
        uid = uuid4().hex
        Key = f"{self.cosConf['fileDirName']}/{uid}.{suffix}"
        Url = f"{self.cosConf['baseUrl']}/{Key}"
        print("flag, msg:", flag, msg)
        thumbnailKey = None
        PicOperations = ''
        length = len(Content)
        print("length:", length/1024)
        if flag and length > self.IMG_THUMBNAIL_SIZE:
            thumbnailKey = f"{uid}-tt.{suffix}"
            Key = f"{self.cosConf['fileDirName']}/{uid}-o.{suffix}"
            Url = f"{self.cosConf['baseUrl']}/{self.cosConf['fileDirName']}/{thumbnailKey}"
            scale = self.getCompressSize(length=length)
            v = rule or f"imageMogr2/thumbnail/!{scale}p"
            PicOperations = '{"is_pic_info":1,"rules":[{"fileid":"%s","rule":"%s", "bucket":"%s"}]}' % (thumbnailKey, v, self.cosConf['bucketName'])
        return dict(
            url=Url,
            key=Key,
            thumbnailKey=thumbnailKey,
            PicOperations=PicOperations
        )

    def getCompressSize(self, length):
        return str(int(round(self.IMG_THUMBNAIL_SIZE/length, 2)*100+15))




    def _uploadFile(self, FileName, Content:bytes, ContentMD5 = None, **kwargs):
        # PicOperations={"is_pic_info":1,"rules":[{"fileid":"format.jpg","rule":"imageView2/format/png"}]}
        # imageMogr2/thumbnail/!".concat(scale.toString()).concat("p")
        # newFileName.concat("-tt").concat(".").concat(suffixName)
        ret = {}
        if len(Content) > self.IMG_MAX_SIZE:
            return {
                'fileName': FileName,
                'url': "",
                'msg': '图片大于5M'
            }
        # ContentMD5 md5字符串
        if ContentMD5:
            kwargs['ContentMD5'] = ContentMD5
        rule = 'rule' in kwargs and kwargs.pop('rule')
        info = self.getInfo(FileName, Content, rule=rule)
        """
        url=Url,
        key=Key,
        thumbnailKey=thumbnailKey,
        PicOperations=PicOperations
        """
        Key = info.get('key')
        url = info.get('url')

        if info.get('PicOperations'):
            res = self.client.ci_put_object(self.cosConf['bucketName'], Body=Content,Key=Key,PicOperations=info['PicOperations'],**kwargs)
        else:
            res = self.client.put_object(self.cosConf['bucketName'], Body=Content,Key=Key,**kwargs)
        print("返回结果:", res)
        if isinstance(res, tuple):
            res = res[0]
        if not res.get('ETag'):
            url = ""
        res = {
            'fileName': FileName,
            'url': url
        }
        return res

    def uploadFile(self, FilePath:str, ContentMD5 = None, **kwargs):
        with open(FilePath, 'rb') as f:
            Content = f.read()

        FileName = FilePath.split('/')[-1]

        res = self._uploadFile(FileName, Content, ContentMD5=ContentMD5, **kwargs)
        print("res:", res)
        return res



    @classmethod
    def isImage(cls, filePath=None, top32Bit:bytes=None):
        if not filePath and not top32Bit:
            return False, '参数不能都为空'
        if not top32Bit and os.path.exists(filePath):
            with open(filePath, 'rb') as f:
                top32Bit = f.read(32)
        try:
            res = imghdr.what(filePath, top32Bit)
        except FileNotFoundError as e:
            return False, e.strerror
        if res:
            return True, res
        else:
            return False,'不是图片'


if __name__ == '__main__':
    # PicOperations='{"is_pic_info":1,"rules":[{"fileid":"format.jpg","rule":"imageView2/format/png"}]}'
    # imageMogr2/format/webp
    cosOp = CosOp({})
    cosOp.uploadFile('./test.jfif', rule='imageMogr2/format/webp')


import asyncio
import re

from lxml import etree
from functools import reduce
from CrawlSpider.Utils.SpiderRequest import spiderRequest,webSplash
from CrawlSpider.Utils.Helper import  getRealUrl
from CrawlSpider.Utils.DateUtils import  DateUtils

class BaseNewsSpider:

    def __init__(self, seed, listXPaths=None, detailXPaths=None, **kwargs):
        self.seed = seed
        self.listXPaths = listXPaths or {}
        self.detailXPaths = detailXPaths or {}
        self.kwargs = kwargs
        self.fieldRegExp = {}
        self.initData()


    def initData(self, **kwargs):
        self.listKw = kwargs.get('listKw') or self.kwargs.get('list') or {}
        self.detailKw = kwargs.get('detailKw') or self.kwargs.get('detail') or {}

    def updateListXPath(self, XPaths):
        self.listXPaths.update(XPaths)

    def updateDetailXPath(self, XPaths):
        self.detailXPaths.update(XPaths)

    #  修改  只允许请求为GET或POST
    async def getContent(self, url=None, **kwargs):
        url = url or self.seed
        res = None
        # isAllow302=True, isUsingProxy=False, isSplash=False
        method = ('method' in kwargs and kwargs.pop('method')) or 'GET'

        if method.upper() == 'GET':
            res = await spiderRequest.get(url, **kwargs)
        elif method.upper() == 'POST':
            res = await spiderRequest.post(url, **kwargs)
        # print("res:", res)

        return res['content']

    #  修改  只允许请求为GET或POST
    async def getTree(self, url=None, **kwargs) -> etree._Element:
        tree = None
        encoding = ('encoding' in kwargs and kwargs.pop('encoding')) or 'utf-8'
        res = await self.getContent(url, **kwargs)
        if res:
            result = await self.preProcess(res.decode(encoding=encoding, errors='ignore'))
            tree = etree.HTML(result)
        return tree

    @classmethod
    async def preProcess(cls, htmlStr):
        charList = ['&nbsp;', '&amp;', '&lt;', '&gt;', '&quot;', '&qpos;']
        for charStr in charList:
            if charStr == '&amp;':
                htmlStr = htmlStr.replace(charStr, '&')
            else:
                htmlStr = htmlStr.replace(charStr, '')

        return htmlStr


    async def getList(self, **kwargs):
        """
        [{'url': 'xxxx},{},{}....]
        :return:
        """
        items = []
        parentNodes, XPaths = await self.getParentNodesAndXPaths(self.seed, self.listXPaths, **self.listKw)
        # print("getList:", parentNodes, XPaths)
        if parentNodes:
            for parentNode in parentNodes:
                item = await self.getFields(parentNode, XPaths)
                items.append(item)

        return items

    async def getFields(self, parentNode, XPaths, isList=1):
        item = {}
        for fieldName in XPaths:
            value: dict = XPaths[fieldName]
            fieldMatchExp = value.get('fieldMatchExp')
            fieldValue = []
            vType = value.get('vType')
            typ = 'txtNoTag'
            v = None
            if vType:
                typ, v = vType
            if typ == 're':
                fieldValue = [await self.getFieldByRe(fieldName, fieldMatchExp,v, etree.tostring(parentNode, with_tail=False, encoding='utf-8',
                                                    method='html').decode())]
            else:
                fieldElems = parentNode.xpath(fieldMatchExp) if fieldMatchExp else [parentNode]
                if fieldElems:
                    isList = int(isList)
                    patch = []
                    if isList > len(fieldElems):
                        patch = [None] * (isList - len(fieldElems))
                        isList = -1

                    if isList == 1:
                        fieldElem = fieldElems[0]
                        fieldValue = [self.getFieldValue(fieldElem, typ, v)]
                    elif isList == -1:
                        fieldValue = []
                        for fieldElem in fieldElems:
                            fieldValue.append(self.getFieldValue(fieldElem, typ, v))
                    else:
                        fieldValue = []
                        for i in range(isList):
                            fieldElem = fieldElems[i]
                            fieldValue.append(self.getFieldValue(fieldElem, typ, v))
                    fieldValue.extend(patch)

            if isinstance(fieldValue, list) and len(fieldValue):
                if len(fieldValue) == 1:
                    item.setdefault(fieldName, fieldValue[0])
                else:
                    for i in range(len(fieldValue)):
                        item.setdefault(f"{fieldName}{i+1}", fieldValue[i])

        return item

    def getFieldValue(self, fieldElem, typ, v):
        fieldValue = None
        if typ == 'txtAndTag':
            fieldValue = etree.tostring(fieldElem, with_tail=False, encoding='utf-8',
                                        method='html').decode()
        elif typ == 'txtNoTag':
            fieldValue = fieldElem.xpath('string(.)')
            fieldValue = fieldValue and fieldValue.strip()
        elif typ == 'attr' and v:
            fieldValue = fieldElem.attrib.get(v)
        return fieldValue

    async def getFieldByRe(self, fieldName, regExp, groupNum, content):
        if content:
            try:
                groupNum = int(groupNum) if groupNum else 0
            except:
                groupNum = 0
            expInfo = self.fieldRegExp.get(fieldName)
            if expInfo and expInfo.get('exp') == regExp:
                _regExp = expInfo.get('regExp')
            else:
                _regExp = re.compile(regExp)
                self.fieldRegExp.update({
                    fieldName: {
                        'exp': regExp,
                        'regExp': _regExp
                    }
                })
            result = re.search(_regExp, content)
            return result.group(groupNum)

    async def getParentNodesAndXPaths(self,url, XPaths, **kwargs):
        tree = await self.getTree(url=url, **kwargs)
        if tree is None:
            return None, None
        prefix = XPaths.get('prefix')
        parentNodes = [tree]
        if prefix:
            XPaths = dict([(k, XPaths[k]) for k in XPaths if k != 'prefix'])
            parentNodes = tree.xpath(prefix)
        return parentNodes, XPaths

    async def getParentNodesAndXPathsByContent(self,content, XPaths, **kwargs):
        tree = await self.getTreeByContent(content)
        if tree is None:
            return None, None
        prefix = XPaths.get('prefix')
        parentNodes = [tree]
        if prefix:
            XPaths = dict([(k, XPaths[k]) for k in XPaths if k != 'prefix'])
            parentNodes = tree.xpath(prefix)
        return parentNodes, XPaths

    async def getTreeByContent(self, text:str):
        if text:
            return etree.HTML(text)

    async def getDetail(self, item:dict):
        fieldDic = {}
        print("item:", item)
        url = item.get('url')
        if url:
            detailUrl = await self.getDetailUrl(url)
            if detailUrl != url:
                item['url'] = detailUrl
            self.detailKw.update({'headers':{'Referer': self.seed}})
            parentNodes, XPaths = await self.getParentNodesAndXPaths(detailUrl, self.detailXPaths, **self.detailKw)
            # print("detailKw:", parentNodes, XPaths)
            if parentNodes:
                # isList = False if 'isList' not in self.detailKw else self.detailKw.pop('isList')
                fieldDic = await self.getFields(parentNodes[0], XPaths, isList=self.kwargs.get('isList', 1))
        return fieldDic

    async def getDetailUrl(self, url):
        return getRealUrl(url, self.seed)

    async def transfer(self,fieldDic: dict, **kwargs):
        dateTime = fieldDic.get('publish_time')
        dateTimeFormat = kwargs.get('dateTimeFormat')
        fieldDic.update({
            'publish_time': DateUtils.dateTimeToStr(DateUtils.strToDateTime(dateTime, dateTimeFormat))
        })
        return fieldDic

    async def startSpider(self):
        urlList = await self.getList()
        urlList = self.LinkDeduplication(urlList)
        isSplash = self.detailKw.get('isSplash')
        if isSplash:
            for urlItem in urlList:
                await self.getResult(urlItem)
        else:
            tasks = [asyncio.create_task(self.getResult(urlItem)) for urlItem in urlList]
            await asyncio.gather(*tasks)
        try:
            if webSplash.driver:
                # 关闭浏览器
                webSplash.driver.quit()
        except Exception as e:
            print("e:", e)


    async def startSpiderByThread(self):
        urlList = await self.getList()
        urlList = self.LinkDeduplication(urlList)
        for urlItem in urlList:
            await self.getResult(urlItem)
        try:
            if webSplash.driver:
                webSplash.driver.close()
        except Exception as e:
            print("e:", e)




    def LinkDeduplication(self, itemList):
        run_function = lambda x, y: x if y in x else x + [y]
        d = reduce(run_function, [[]] + itemList)
        return d

    async def getResult(self, urlItem):
        itemDic = await self.getDetail(urlItem)
        if itemDic:
            urlItem.update(itemDic)
            fieldDic = await self.transfer(urlItem)
            combineItem = await self.combineField(fieldDic)
        else:
            combineItem = itemDic
        await self.saveTo(combineItem)


    async def combineField(self,item:dict):
        combineItem = {}
        ks, kMap = self.keyMap()
        isBlank = self.kwargs.get('isBlank', False)
        for k in ks:
            mapKey = kMap.get(k)
            if item.get(k) or isBlank:
                combineItem.setdefault(mapKey or k, item.get(k))
        return combineItem


    def getKMap(self):
        return {
            'url': 'snapshotUrl',
            'content': 'new_content'
        }

    def keyMap(self):
        ks =  "url,title,publish_time,author,content,source".split(',')
        kMap = self.getKMap()
        return ks, kMap

    async def saveTo(self, resultItem):
        pass





# -*- coding: utf-8 -*-
# __author__ : Ricky
# __createTime__ : 2022/7/8 9:42
# __fileName__ : CrawlSpider taskUtils.py
# __devIDE__ : PyCharm
import platform
import asyncio


def initEventLoop():
    sysType = platform.system()
    if sysType == 'Linux':
        try:
            import uvloop
            asyncio.set_event_loop(uvloop)
        except:
            pass
    return asyncio.get_event_loop()


def executeTask(ft: asyncio.Future):
    loop = initEventLoop()
    loop.run_until_complete(ft)

async def launchTasks(taskItems):
    tasks = [
        asyncio.create_task(item['fn'](**item['params']))
        for item in taskItems
    ]
    await asyncio.gather(*tasks)
    return tasks








"""
This file contains some modification aimed at fixing bugs and improving the dateinfer library.
The newlines/modification line are preceded with comments indicating the purpose.
"""

import collections
import itertools
import string

from CrawlSpider.date_infer.date_elements import (
    AMPM,
    DayOfMonth,
    Filler,
    Hour12,
    Hour24,
    Minute,
    MonthNum,
    MonthTextLong,
    MonthTextShort,
    Second,
    Timezone,
    UTCOffset,
    WeekdayLong,
    WeekdayShort,
    Year2,
    Year4,
)
from CrawlSpider.date_infer.ruleproc import (
    And,
    Contains,
    Duplicate,
    If,
    KeepOriginal,
    Sequence,
    Swap,
    SwapDuplicateWhereSequenceNot,
    SwapSequence,
)

# DATE_ELEMENTS is an ordered sequence of date elements, excluding the filler. It is ordered
# in descending "restrictivity".
# The order is a little loose since date element domains do not necessarily overlap (e.g., the
# range of Jan .. Dec is 12, but the domain is independent of hours 0 .. 23), but overall a lesser
# value should be preferred over a greater value.
# The RULES will be applied after the list is generated following these precedence rules.
DATE_ELEMENTS = (
    AMPM(),
    MonthNum(),
    Hour12(),
    Hour24(),
    DayOfMonth(),
    Minute(),
    Second(),
    Year2(),
    Year4(),
    UTCOffset(),
    MonthTextShort(),
    MonthTextLong(),
    WeekdayShort(),
    WeekdayLong(),
    Timezone(),
)

F = Filler  # short-hand to clarify rules
RULES = [
    If(Sequence(Year4, Year2), SwapSequence([Year4, Year2], [Year4, MonthNum])),
    If(
        Sequence(MonthNum, F("/"), r"\d", F("/"), Year4),
        SwapSequence(
            [MonthNum, F("/"), r"\d", F("/"), Year4],
            [MonthNum, F("/"), DayOfMonth, F("/"), Year4],
        ),
    ),
    If(
        Sequence(MonthNum, F("/"), r"\d", F("/"), Hour24),
        SwapSequence(
            [MonthNum, F("/"), r"\d", F("/"), Hour24],
            [MonthNum, F("/"), DayOfMonth, F("/"), Year2],
        ),
    ),
    If(
        Sequence(MonthNum, F("-"), r"\d", F("-"), Hour24),
        SwapSequence(
            [MonthNum, F("-"), r"\d", F("-"), Hour24],
            [MonthNum, F("-"), DayOfMonth, F("-"), Year2],
        ),
    ),
    If(
        Sequence(MonthNum, F("/"), r"\d", F("/"), MonthNum),
        SwapSequence(
            [MonthNum, F("/"), r"\d", F("/"), MonthNum],
            [MonthNum, F("/"), DayOfMonth, F("/"), Year2],
        ),
    ),
    If(
        Sequence(MonthNum, F("-"), r"\d", F("-"), MonthNum),
        SwapSequence(
            [MonthNum, F("-"), r"\d", F("-"), MonthNum],
            [MonthNum, F("-"), DayOfMonth, F("-"), Year2],
        ),
    ),
    If(
        Sequence(MonthNum, F(":"), r"\d", F(":"), r"\d"),
        SwapSequence(
            [MonthNum, F(":"), r"\d", F(":"), r"\d"],
            [Hour12, F(":"), Minute, F(":"), Second],
        ),
    ),
    If(
        Sequence(Hour24, F(":"), r"\d", F(":"), r"\d"),
        SwapSequence(
            [Hour24, F(":"), r"\d", F(":"), r"\d"],
            [Hour24, F(":"), Minute, F(":"), Second],
        ),
    ),
    If(
        Sequence(MonthNum, F(":"), r"\d", r"\D"),
        SwapSequence([MonthNum, F(":"), "."], [Hour12, F(":"), Minute]),
    ),
    If(
        Sequence(Hour24, F(":"), r"\d", r"\D"),
        SwapSequence([Hour24, F(":"), r"\d"], [Hour24, F(":"), Minute]),
    ),
    If(
        Sequence(MonthNum, F(":"), r"\d"),
        SwapSequence([MonthNum, F(":"), "."], [Hour24, F(":"), Minute]),
    ),
    If(
        And(Sequence(Hour12, F(":"), Minute), Contains(Hour24)),
        Swap(Hour24, DayOfMonth),
    ),
    If(
        And(Sequence(Hour12, F(":"), Minute), Duplicate(Hour12)),
        SwapDuplicateWhereSequenceNot(Hour12, MonthNum, (Hour12, F(":"))),
    ),
    If(
        And(Sequence(Hour24, F(":"), Minute), Duplicate(Hour24)),
        SwapDuplicateWhereSequenceNot(Hour24, DayOfMonth, [Hour24, F(":")]),
    ),
    If(Contains(MonthNum, MonthTextLong), Swap(MonthNum, DayOfMonth)),
    If(Contains(MonthNum, MonthTextShort), Swap(MonthNum, DayOfMonth)),
    If(
        Sequence(MonthNum, ".", Hour12),
        SwapSequence([MonthNum, ".", Hour12], [MonthNum, KeepOriginal, DayOfMonth]),
    ),
    If(
        Sequence(MonthNum, ".", Hour24),
        SwapSequence([MonthNum, ".", Hour24], [MonthNum, KeepOriginal, DayOfMonth]),
    ),
    If(
        Sequence(Hour12, ".", MonthNum),
        SwapSequence([Hour12, ".", MonthNum], [DayOfMonth, KeepOriginal, MonthNum]),
    ),
    If(
        Sequence(Hour24, ".", MonthNum),
        SwapSequence([Hour24, ".", MonthNum], [DayOfMonth, KeepOriginal, MonthNum]),
    ),
    If(Duplicate(MonthNum), Swap(MonthNum, DayOfMonth)),
    If(Sequence(F("+"), Year4), SwapSequence([F("+"), Year4], [UTCOffset, None])),
    If(
        Sequence(Second, F("-"), Year4),
        SwapSequence([Second, F("-"), Year4], [Second, UTCOffset, None]),
    ),
    If(
        Sequence(Minute, F("-"), Year4),
        SwapSequence([Minute, F("-"), Year4], [Minute, UTCOffset, None]),
    ),
    If(
        Sequence(Hour24, ".", r"\D"),
        SwapSequence([Hour24, ".", r"\D"], [DayOfMonth, KeepOriginal, KeepOriginal]),
    ),
    If(
        Sequence(DayOfMonth, ".", MonthNum, ".", DayOfMonth),
        SwapSequence(
            [DayOfMonth, ".", MonthNum, ".", DayOfMonth],
            [DayOfMonth, KeepOriginal, MonthNum, KeepOriginal, Year2],
        ),
    ),
    If(
        And(Duplicate(Minute), Contains(Hour24)),
        SwapDuplicateWhereSequenceNot(Minute, Second, [Minute]),
    ),
    If(And(Duplicate(DayOfMonth), Contains(MonthNum)), Swap(MonthNum, Year2)),
    If(
        Duplicate(DayOfMonth),
        SwapDuplicateWhereSequenceNot(DayOfMonth, MonthNum, [DayOfMonth]),
    ),
]


_alt_directives = {'%z': '%Y',
                   '%Y': '%z'}


def infer(examples, alt_rules=None):
    """
    Returns a datetime.strptime-compliant format string for parsing the *most likely* date format
    used in examples. examples is a list containing example date strings.
    """
    date_classes = _tag_most_likely(examples)

    if alt_rules:
        date_classes = _apply_rewrites(date_classes, alt_rules)
    else:
        date_classes = _apply_rewrites(date_classes, RULES)

    date_string = ""
    directives = []
    for date_class in date_classes:
        directive = date_class.directive
        if '%' in directive:
            if directive in directives:
                directive = _alt_directives.get(directive, directive)
            if directive in directives:
                continue
        directives.append(directive)
        date_string += directive

    return date_string


def _apply_rewrites(date_classes, rules):
    """
    Return a list of date elements by applying rewrites to the initial date element list
    """
    for rule in rules:
        date_classes = rule.execute(date_classes)

    return date_classes


def _mode(elems):
    """
    Find the mode (most common element) in list elems. If there are ties, this function returns the
    least value.

    If elems is an empty list, returns None.
    """
    if not elems:
        return None

    c = collections.Counter()
    c.update(elems)

    most_common = c.most_common(1)
    most_common.sort()
    return most_common[0][
        0
    ]  # most_common[0] is a tuple of key and count; no need for the count


def _most_restrictive(date_elems):
    """
    Return the date_elem that has the most restrictive range from date_elems
    """
    most_index = len(DATE_ELEMENTS)
    for date_elem in date_elems:
        if date_elem in DATE_ELEMENTS and DATE_ELEMENTS.index(date_elem) < most_index:
            most_index = DATE_ELEMENTS.index(date_elem)
    if most_index < len(DATE_ELEMENTS):
        return DATE_ELEMENTS[most_index]

    raise KeyError("No least restrictive date element found")


def _percent_match(date_classes, tokens):
    """
    For each date class, return the percentage of tokens that the class matched (floating point
    [0.0 - 1.0]).

    The returned value is a tuple of length patterns. Tokens should be a list.
    """
    match_count = [0] * len(date_classes)

    for i, date_class in enumerate(date_classes):
        for token in tokens:
            if date_class.is_match(token):
                match_count[i] += 1

    percentages = tuple([float(m) / len(tokens) for m in match_count])
    return percentages


def _tag_most_likely(examples):
    """
    Return a list of date elements by choosing the most likely element for a token within examples
    (context-free).
    """
    tokenized_examples = [_tokenize_by_character_class(example) for example in examples]

    # We currently need the tokenized_examples to all have the same length, so drop instances that
    # have a length that does not equal the mode of lengths within tokenized_examples
    token_lengths = [len(e) for e in tokenized_examples]
    token_lengths_mode = _mode(token_lengths)
    tokenized_examples = [
        example for example in tokenized_examples if len(example) == token_lengths_mode
    ]

    # Now, we iterate through the tokens, assigning date elements based on their likelihood.
    # In cases where the assignments are unlikely for all date elements, assign filler.
    most_likely = []
    for token_index in range(0, token_lengths_mode):
        tokens = [token[token_index] for token in tokenized_examples]
        probabilities = _percent_match(DATE_ELEMENTS, tokens)
        max_prob = max(probabilities)
        if max_prob < 0.5:
            most_likely.append(Filler(_mode(tokens)))
        else:
            if probabilities.count(max_prob) == 1:
                most_likely.append(DATE_ELEMENTS[probabilities.index(max_prob)])
            else:
                choices = []
                for index, prob in enumerate(probabilities):
                    if prob == max_prob:
                        choices.append(DATE_ELEMENTS[index])
                most_likely.append(_most_restrictive(choices))

    return most_likely


def _tokenize_by_character_class(s):
    """
    Return a list of strings by splitting s (tokenizing) by character class.

    For example:
    _tokenize_by_character_class('Sat Jan 11 19:54:52 MST 2014') =>
        ['Sat', ' ', 'Jan', ' ', '11', ' ', '19', ':', '54', ':', '52', ' ', 'MST', ' ', '2014']

    _tokenize_by_character_class('2013-08-14') => ['2013', '-', '08', '-', '14']
    """
    # Callables per character class. Return True/False depending on whether the character is in the
    # respective class.
    character_classes = [
        lambda x: x.isdigit(),
        lambda x: x.isalpha(),
        lambda x: x in string.punctuation,
        lambda x: x.isspace(),
    ]

    result = []
    rest = list(s)
    while rest:
        progress = False
        for part_of_class in character_classes:
            if part_of_class(rest[0]):
                progress = True
                token = ""
                for take_away in itertools.takewhile(part_of_class, rest[:]):
                    token += take_away
                    rest.pop(0)
                result.append(token)
                break
        if (
            not progress
        ):  # none of the character classes matched; unprintable character?
            result.append(rest[0])
            rest = rest[1:]

    return result




from fake_useragent import UserAgent

from CrawlSpider.Utils.Helper import initUaCache


class RetrieveUA:

    ua = None

    @classmethod
    def getRandomUa(cls, **kwargs):
        if not cls.ua:
            cls.init(**kwargs)
        return cls.ua.random

    @classmethod
    def init(cls, **kwargs):
        """
        可以自定义 uaFile文件的路径
        """
        initUaCache(**kwargs)
        cls.ua = UserAgent()







import aiofiles
from urllib.parse import urlparse
from urllib3 import encode_multipart_formdata
from typing import AnyStr,List
from CrawlSpider.Utils.SpiderRequest import spiderRequest
from CrawlSpider.Utils.Helper import md5_text
from CrawlSpider.cos_sdk.TencentCloud.CosOp import CosOp


class FileUpload:

    def __init__(self, cosConf, cosOp=None):
        self.cosOp = cosOp or CosOp(cosConf)



    async def uploadFilesToOss(self, fileList:List[AnyStr]):
        fileDic = {}
        for file in fileList:
            async with aiofiles.open(file, 'rb') as fr:
                f = await fr.read()
            data = await self.uploadFileToOss(file, f)
            fileDic.setdefault(file, data)
        return fileDic

    async def uploadFileToOss(self, fileName,fileData:bytes, ContentMD5=None, **kwargs):
        if not fileData:
            res = {
                'fileName': fileName,
                'url': ""
            }
        else:
            res = self.cosOp._uploadFile(fileName, fileData, ContentMD5=ContentMD5, **kwargs)
        print("res:", res)
        return res

    async def _uploadFileToOss(self, fileName,fileData:bytes, ossUploadApi=""):
        data = {
            'file': (fileName, fileData)
        }
        if fileData:
            encode_data = encode_multipart_formdata(data)
            print(encode_data[1])
            res = await spiderRequest.post(ossUploadApi, headers={'Content-Type': encode_data[1]},
              data=encode_data[0], form="json", verify_ssl=False)
            print("res", res)
        else:
            res = None

        if res:
            return res['data']
        else:
            return {
                "fileName": fileName,
                "url": None
            }

    async def uploadFileFromNetToOss(self, url):
        parseResult = urlparse(url)
        # 这里可能有问题，后续再优化
        ext = parseResult.query.lstrip('wx_fmt=')
        uniqueFlag = parseResult.path.split('/')[-2]
        fileName = f"{uniqueFlag}.{ext}"
        print(f"当前图片名称为:{fileName}")
        res = await spiderRequest.get(url)
        data = await self.uploadFileToOss(fileName, res)
        return data

    async def uploadFileFromNetToOssCommon(self, url, fileName=None):
        fileName = fileName or md5_text(url)+'.jpeg'
        print(f"当前图片名称为:{fileName}")
        res = await spiderRequest.get(url)
        data = await self.uploadFileToOss(fileName, res)
        return data



if __name__ == '__main__':
    import asyncio
    fileUpload = FileUpload({})
    loop = asyncio.get_event_loop()
    url = "https://mmbiz.qpic.cn/mmbiz_jpg/rCOKPvQb9vvN0c0ttdLIta9f73N0MRibNWhiaGqhWcufjKpJ51qn5Uqcjic0Js9p25SkezkG45caFoNia6Jmr6cJhg/640?wx_fmt=jpeg"
    s = loop.create_task(fileUpload.uploadFileFromNetToOss(url))
    loop.run_until_complete(s)





from typing import Tuple

from CrawlSpider.Utils.FileUpload import FileUpload, urlparse, List
from bs4 import BeautifulSoup,Tag
import asyncio


class ImgSrcReplace:
    @classmethod
    async def wxgzh(cls, body:str='', soup:Tag=None, fileUpload: FileUpload = None)->Tag:
        """
        body和soup两者必须有一个有值
        :param body: tag的字符串形式
        :param soup: Tag类型的节点
        :param fileUpload: FileUpload 用于将文件资源上传到云存储
        :return: 返回节点字符串形式
        """
        soup = soup or BeautifulSoup(body, 'lxml')

        imgs: List[Tag] = soup.find_all(name='img')
        for img in imgs:
            data_src = img.get('data-src')
            # 判空操作
            if not data_src:
                continue
            data = await fileUpload.uploadFileFromNetToOss(data_src)
            url = data.get('url')
            if url:
                img.attrs.pop('data-src')
                img.attrs.update({
                    'src': url
                })
        return soup

    @classmethod
    async def common(cls, body: str = '', imgAttrName:str='src', sleep=0.05, imgFilePattern=None, imgFileScope:str=None, fileUpload: FileUpload = None) -> \
    Tuple[str, str]:
        """
        body和soup两者必须有一个有值
        :param body: tag的字符串形式
        :param fileUpload: FileUpload 用于将文件资源上传到云存储
        :return: 返回节点字符串形式
        """
        soup = BeautifulSoup(body, 'lxml')
        soup = await LinkHrefReplace.common(soup=soup)
        imgs: List[Tag] = soup.find_all(name='img')
        for img in imgs:
            data_src:str = img.get(imgAttrName)
            if not data_src:
                continue
            imgUrl = data_src
            res = urlparse(data_src)

            if '192.168' not in res.netloc:
                if not res.netloc and not imgFileScope:
                    return body, "图片地址非绝对路径，请传imgFileScope前缀"
                if data_src.startswith('/'):
                    result = urlparse(imgFileScope)
                    imgUrl = f"{result.scheme}://{result.netloc}{data_src}"
                else:
                    # suffix = listUrl.split('/')[-1]
                    # prefix = listUrl.rstrip(suffix)
                    # realUrl = prefix + detailUrl
                    # url.rstrip(url.split('/')[-1])
                    if data_src.startswith('.'):
                        imgUrl = imgFileScope + data_src
                    if not res.scheme and data_src.startswith("//"):
                        scheme = urlparse(imgFileScope).scheme
                        imgUrl = f'{scheme}:{data_src}'
                data = await fileUpload.uploadFileFromNetToOssCommon(imgUrl)
            else:
                data = {
                    'url': None
                }
            realUrl = data.get('url') or ''
            img.attrs.update({
                'src': realUrl,
                'data-src': '',
                'oldsrc': ''
            })
            await asyncio.sleep(delay=sleep)
        return soup.decode(),"图片和链接置换成功"

    @classmethod
    async def delTagsExceptImg(cls, htmlStr: str = None, soup=None, imgAttr='src'):
        soup = soup or BeautifulSoup(htmlStr or "", 'lxml')
        imgs = soup.find_all(name='img')
        for img in imgs:
            img: Tag
            data_src = img.get(imgAttr)
            if not data_src:
                continue
            s = f'###p***###img src="{data_src}"***###/p***'
            img.append(s)
        # 文本都包在p标签中
        res = soup.get_text(separator="<p>", strip=True)
        res = res.strip('<p>')
        # 实现首行缩进 style="text-indent:2em"
        res = res.replace('<p>', '</p><p>')
        res = res.replace('###', '<')
        res = res.replace('***', '>')
        # return f"<p>{res}</p>"
        return res

class LinkHrefReplace:
    @classmethod
    async def common(cls, body:str='', soup:Tag=None)->Tag:
        """
        body和soup两者必须有一个有值
        :param body: tag的字符串形式
        :param soup: Tag类型的节点
        :return: 返回节点字符串形式
        """
        soup = soup or BeautifulSoup(body, 'lxml')
        aLinks: List[Tag] = soup.find_all(name='a')
        for link in aLinks:
            href = link.get('href')
            if href:
                link['href'] = ''
        return soup


class VideoSrcReplace:
    @classmethod
    async def wxgzh(cls, body:str='', soup:Tag=None, fileUpload: FileUpload = None)->str:
        """
        body和soup两者必须有一个有值
        :param body: tag的字符串形式
        :param soup: Tag类型的节点
        :param fileUpload: FileUpload 用于将文件资源上传到云存储
        :return: 返回节点字符串形式
        """
        soup = soup or BeautifulSoup(body, 'lxml')

        imgs: List[Tag] = soup.find_all(name='img')
        for img in imgs:
            data_src = img.get('data-src')
            data = await fileUpload.uploadFileFromNetToOss(data_src)
            url = data.get('url')
            if url:
                img.attrs.pop('data-src')
                img.attrs.update({
                    'src': url
                })
        return soup.decode()


# -*- coding: utf-8 -*-
# __author__ : Ricky
# __createTime__ : 2022/7/8 10:03
# __fileName__ : CrawlSpider twineUtils.py
# __devIDE__ : PyCharm
from twine.commands.upload import upload as twine_upload
from twine.settings import Settings as TwineSetting
from twine.commands.check import check as twine_check
from twine.commands.register import register as twine_register
import datetime
import fire
import os
"""
[options.entry_points]
twine.registered_commands = 
	check = twine.commands.check:main
	upload = twine.commands.upload:main
	register = twine.commands.register:main
console_scripts = 
	twine = twine.__main__:main
"""


def build(username=None, password=None, version=None, distDir=None, is_publish=False, **kwargs):

    if distDir:
        projectDir = os.path.join(distDir, '../')
    else:
        projectDir = os.getcwd()
        distDir = os.path.join(projectDir, 'dist')
    print(projectDir, distDir)
    about = {}

    with open(os.path.join(projectDir, "__version__.py"), "r", encoding="utf-8") as f:
        exec(f.read(), about)
    if version:
        about['__version__'] = version
        _buildVersion_ = datetime.datetime.strftime(datetime.datetime.strptime(version.replace('.', ':'), '%H:%M:%S'), '%H:%M:%S')
        buildVersion = f"0x{_buildVersion_.replace(':', '')}"
        about['__build__'] = buildVersion
        with open(os.path.join(projectDir, "__version__.py"), "w", encoding="utf-8") as f:
            _about = {
                k: (k == '__version__' and version) or (k == '__build__' and buildVersion) or about[k]
                for k in about if k not in {'__builtins__'}
            }
            newLines = [
                f'{k} = "{_about[k]}"' if k != '__build__' else f'{k} = {_about[k]}'
                for k in _about
            ]
            f.write('\n\n')
            f.write('\n'.join(newLines))
            f.write('\n\n')


    os.system(f"python {os.path.join(projectDir, 'setup.py')} sdist bdist_wheel")
    if is_publish:


        version = version or about['__version__']


        dists = [os.path.join(distDir, filePath) for filePath in os.listdir(distDir) if version in filePath]

        twine_upload(
            TwineSetting(
                username=username,
                password=password,
                **kwargs
            ),
            dists
        )





if __name__ == '__main__':
    fire.Fire({
        'build': build
    })












import traceback
import aiomysql
import pymysql




class MySQLDb:
    """A lightweight wrapper around aiomysql.Pool for easy to use
    """
    def __init__(self, host, user, password,
                 db=None,
                 loop=None, sanic=None,
                 minsize=3, maxsize=5,
                 return_dict=True,
                 pool_recycle=7*3600,
                 autocommit=True,
                 charset = "utf8mb4", **kwargs):
        '''
        kwargs: all parameters that aiomysql.connect() accept.
        '''
        self.db_args = {
            'host': host,
            'db': db,
            'user': user,
            'password': password,
            'minsize': minsize,
            'maxsize': maxsize,
            'charset': charset,
            'loop': loop,
            'autocommit': autocommit,
            'pool_recycle': pool_recycle,
        }
        self.sanic = sanic
        # if sanic:
        #     sanic.db = self
        if return_dict:
            self.db_args['cursorclass']=aiomysql.cursors.DictCursor
        if kwargs:
            self.db_args.update(kwargs)
        self.pool = None

    async def init_pool(self):
        if self.sanic:
            self.db_args['loop'] = self.sanic.loop
        self.pool = await aiomysql.create_pool(**self.db_args)

    @staticmethod
    def flat(updates, condition=False, sep=','):
        upsets = []
        values = []
        for k, v in updates.items():
            s = '%s=%%s' % k
            if condition:
                s = "%s='%s'" % (k, v)
                if isinstance(v, tuple):
                    s = f"{k} in {v}"
                elif isinstance(v, (int, float)):
                    s = f"{k} = {v}"
                elif 'null' in v.lower():
                    s = f"{k} is {v}"

            upsets.append(s)
            values.append(v)
        upsets = sep.join(upsets)

        return upsets, values

    async def fetchall(self, query, *parameters, **kwparameters):
        """Returns a row list for the given query and parameters."""
        if not self.pool:
            await self.init_pool()
        async with self.pool.acquire() as conn:
            async with conn.cursor() as cur:
                try:
                    await cur.execute(query, kwparameters or parameters or None)
                    ret = await cur.fetchall()
                except pymysql.err.InternalError:
                    await conn.ping()
                    await cur.execute(query, kwparameters or parameters or None)
                    ret = await cur.fetchall()
                return ret

    async def fetchone(self, query, *parameters, **kwparameters):
        """Returns the (singular) row returned by the given query.
        """
        if not self.pool:
            await self.init_pool()
        async with self.pool.acquire() as conn:
            async with conn.cursor() as cur:
                try:
                    await cur.execute(query, kwparameters or parameters)
                    ret = await cur.fetchone()
                except pymysql.err.InternalError:
                    await conn.ping()
                    await cur.execute(query, kwparameters or parameters)
                    ret = await cur.fetchone()
                return ret

    async def execute(self, query, *parameters, **kwparameters):
        """Executes the given query, returning the lastrowid from the query."""
        if not self.pool:
            await self.init_pool()
        async with self.pool.acquire() as conn:
            async with conn.cursor() as cur:
                try:
                    await cur.execute(query, kwparameters or parameters or None)

                except Exception:
                    # https://github.com/aio-libs/aiomysql/issues/340
                    await conn.ping()
                    await cur.execute(query, kwparameters or parameters or None)
                return cur.lastrowid

    # executemany
    async def executeMany(self, query, items):
        """Executes the given query, returning the lastrowid from the query."""
        if not self.pool:
            await self.init_pool()
        async with self.pool.acquire() as conn:
            async with conn.cursor() as cur:
                try:
                    await cur.executemany(query, items)
                except Exception:
                    # https://github.com/aio-libs/aiomysql/issues/340
                    await conn.ping()
                    await cur.executemany(query, items)
                return cur.lastrowid
    # high level interface
    async def table_has(self, table_name, conditions):
        """
        judge record that  is or not in table
        :param table_name: table to fetching
        :param conditions: fetch condtions
        :return:
        """
        condition, _ = self.flat(conditions, condition=True, sep=' and ')
        query = f'SELECT * FROM {table_name} WHERE {condition}'
        res = await self.fetchone(query)
        if res and len(res):
            return True
        return False

    async def table_insert(self, table_name, item, ignore_duplicated=True):
        '''item is a dict : key is mysql table field'''
        fields = list(item.keys())
        values = list(item.values())
        fieldstr = ','.join(fields)
        valstr = ','.join(['%s'] * len(item))
        sql = 'INSERT INTO %s (%s) VALUES(%s)' % (table_name, fieldstr, valstr)
        try:
            last_id = await self.execute(sql, *values)
            return last_id
        except Exception as e:
            if ignore_duplicated and e.args[0] == 1062:
                # just skip duplicated item
                return 0
            traceback.print_exc()
            print('sql:', sql)

            for i in range(len(fields)):
                vs = str(values[i])
                if len(vs) > 300:
                    print(fields[i], ' : ', len(vs), type(values[i]))
                else:
                    print(fields[i], ' : ', vs, type(values[i]))
            return e

    async def table_insertMany(self, table_name, items):
        '''item is a dict : key is mysql table field'''
        item = items[0]
        fields = list(item.keys())
        values = list(item.values())
        fieldstr = ','.join(fields)
        valstr = ','.join(['%s'] * len(item))
        sql = 'INSERT INTO %s (%s) VALUES(%s)' % (table_name, fieldstr, valstr)
        try:
            data = []
            for item in items:
                data.append(tuple(item.values()))
            last_id = await self.executeMany(sql, data)
            return last_id
        except Exception as e:
            traceback.print_exc()
            print('sql:', sql)

            for i in range(len(fields)):
                vs = str(values[i])
                if len(vs) > 300:
                    print(fields[i], ' : ', len(vs), type(values[i]))
                else:
                    print(fields[i], ' : ', vs, type(values[i]))
            return e


    async def table_update(self, table_name, updates,
                     conditions):
        """
        according to condtions(dict) to update some field (updates) ==> dict
        :param table_name:
        :param updates:
        :param conditions:
        :return:
        """

        upsets, values = self.flat(updates)
        condition, _ = self.flat(conditions, condition=True, sep=' and ')
        sql = 'UPDATE %s SET %s WHERE %s' % (
            table_name,
            upsets,
            condition
        )
        await self.execute(sql, *values)



import asyncio
import copy
import json
import datetime
import requests 


from urllib.parse import urlparse
from CrawlSpider.Utils.SplashHandler import webSplash



headers = {
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.131 Safari/537.36'
}



class MyEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, bytes):
            return obj.decode('utf-8', 'ignore')
        if isinstance(obj, datetime.datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, datetime.date):
            return obj.strftime('%Y-%m-%d')
        else:
            return json.JSONEncoder.default(self, obj)



class SpiderRequestSync:
    def getSync(self,url, form="content", **kwargs):
        _headers: dict = (kwargs.get('headers') and kwargs.pop('headers')) or {}
        _headers.update(headers)
        kwargs.get('chrome_options') and kwargs.pop('chrome_options')
        proxyUrl = (kwargs.get('proxy') and kwargs.pop('proxy')) or ""
        if proxyUrl:
            res = urlparse(proxyUrl)
            proxies = {
                res.scheme: res.netloc
            }
            kwargs['proxies'] = proxies
        resp = requests.get(url, headers=_headers, **kwargs)
        r = None

        if resp and resp.ok:  # 注意这里是status，不是status_code
            if form == 'json':
                r = resp.json()
            elif form == 'text':
                r = resp.text
            elif form == 'content':
                r = resp.content
        return r


    def postSync(self, url, form="content", **kwargs):
        _headers: dict = (kwargs.get('headers') and kwargs.pop('headers')) or {}
        _headers.update(headers)
        kwargs.get('chrome_options') and kwargs.pop('chrome_options')
        proxyUrl = (kwargs.get('proxy') and kwargs.pop('proxy')) or ""
        if proxyUrl:
            res = urlparse(proxyUrl)
            proxies = {
                res.scheme: res.netloc
            }
            kwargs['proxies'] = proxies
        resp =  requests.post(url, headers=_headers, **kwargs)
        r = None
        if resp.ok:  # 注意这里是status，不是status_code
            if form == 'json':
                r = resp.json()
            elif form == 'text':
                r = resp.text
            elif form == 'content':
                r = resp.content
        return r

    def get(self, url, isAllow302=True, isUsingProxy=False, isSplash=False, **kwargs):
        if isUsingProxy:
            ret = self.proxy(self.get_, url, isAllow302=isAllow302, isSplash=isSplash,isAsync=True, **kwargs)
        else:
            ret = self.get_(url, isAllow302=isAllow302, isSplash=isSplash,**kwargs)
        return ret

    def get_(self, url, isAllow302=True,  isSplash=False,isAsync=True, **kwargs):
        if isSplash:
            return self.splash(url, **kwargs)
        elif not isAsync:
            return self.getSync(url, **kwargs)

    @classmethod
    def splash(cls,url, **kwargs):
        loop = asyncio.get_running_loop()
        # --proxy-server=socks5://xxx.xxx:xxx
        options = kwargs.get('chrome_options')
        splashTime = kwargs.get('splashTime') or 0.2
        isUsingNewDriver = kwargs.get('isUsingNewDriver')
        driver = None
        if isUsingNewDriver:
            driver = webSplash.getNewDriver(options)
        elif options:
            driver = webSplash.getDriver(options=options)
        future = loop.run_in_executor(None, webSplash.getSource, url, driver, splashTime)
        return future

    @classmethod
    def splashSync(cls, url, **kwargs):
        options = kwargs.get('chrome_options')
        splashTime = kwargs.get('splashTime') or 0.2
        isUsingNewDriver = kwargs.get('isUsingNewDriver')
        driver = None
        if isUsingNewDriver:
            driver = webSplash.getNewDriver(options)
        elif options:
            driver = webSplash.getDriver(options=options)
        return webSplash.getSource(url, driver, splashTime)


    @classmethod
    def proxy(cls,fn, url, isAllow302=True,  isSplash=False, **kwargs):
        from CrawlSpider.Utils.RetrieveProxy import RetrieveProxy
        _kwargs = copy.deepcopy(kwargs)
        isFree = True
        if 'isFree' in _kwargs:
            isFree = _kwargs.pop('isFree')
        if isFree:
            retry = 3
            proxy = RetrieveProxy.getProxyFree()
            print("获取到的免费代理IP:", proxy)
            # proxies requests {}
            httpProxy = f"http://{proxy}"
            _kwargs.update({
                'proxy': httpProxy,
                'chrome_options': [f'--proxy-server={httpProxy}']
            })
            while retry:
                try:
                    ret = fn(url, isAllow302=isAllow302, isSplash=isSplash, **_kwargs)
                    return ret
                except Exception as e:
                    print("e", e)
                    retry -= 1
            ret = RetrieveProxy.delete(proxy)
            print("删除结果:", ret)
        zhiMaProxy = RetrieveProxy.getProxyByZhiMa()
        if zhiMaProxy:
            print("获取到的收费代理IP:", zhiMaProxy)
            ret = RetrieveProxy.addProxy({
                'source': "zhiMa",
                'proxy': zhiMaProxy
            })
            print("添加结果:", ret)
            httpProxy = f"http://{zhiMaProxy}"
            _kwargs.update({
                'proxy': httpProxy,
                'chrome_options': [f'--proxy-server={httpProxy}']
            })
            try:
                ret = fn(url, isAllow302=isAllow302, isSplash=isSplash, **_kwargs)
                return ret
            except Exception as e:
                print("e", e)
                return None


    def post(self, url, isAllow302=True, isUsingProxy=False, isSplash=False,**kwargs):
        if isUsingProxy:
            ret = self.proxy(self.post_, url, isAllow302=isAllow302, isSplash=isSplash, **kwargs)
        else:
            ret = self.post_(url, isAllow302=isAllow302, isSplash=isSplash, **kwargs)

        return ret

    def post_(self, url, isAllow302=True, isSplash=False,isAsync=True,**kwargs):
        ret = self.postSync(url,  **kwargs)
        return ret

    @classmethod
    def toJson(cls, data,**kwargs):
        return json.dumps(data, ensure_ascii=False, cls=MyEncoder, **kwargs)
    @classmethod
    def fromJson(cls, data, encoding='utf-8', **kwargs):
        return json.loads(data, encoding=encoding, **kwargs)


spiderRequest = SpiderRequestSync()



import uuid
import math
import time
import redis

class RedisDB:

    def __init__(self, config=None, lock_name=None):
        """
        :param config: {"host":"",
                        "port": 0,
                        "db": 0,
                        "password": "",
                        "encoding": "",
                        "decode_responses": False,
                        "max_connections": 1,
                        "target_max_memory": 1024
                        }
        """
        self.config = config
        pool = redis.ConnectionPool(**self.config)
        self.connection_pool = pool
        self.lock_name = lock_name or 'spider'

    def getClient(self):
        client = redis.Redis(connection_pool=self.connection_pool)
        return client

    def acquire_lock_with_timeout(self, conn, acquire_timeout=3, lock_timeout=2):
        """
        基于 Redis 实现的分布式锁
        :param conn: Redis 连接
        :param lock_name: 锁的名称
        :param acquire_timeout: 获取锁的超时时间，默认 3 秒
        :param lock_timeout: 锁的超时时间，默认 2 秒
        :return:
        """

        identifier = str(uuid.uuid4())
        lockname = f'lock:{self.lock_name}'
        lock_timeout = int(math.ceil(lock_timeout))

        end = time.time() + acquire_timeout

        while time.time() < end:
            # 如果不存在这个锁则加锁并设置过期时间，避免死锁
            if conn.set(lockname, identifier, ex=lock_timeout, nx=True):
                return identifier

            time.sleep(0.001)

        return False


    def release_lock(self, conn, identifier):
        """
        释放锁

        :param conn: Redis 连接
        :param lockname: 锁的名称
        :param identifier: 锁的标识
        :return:
        """
        unlock_script = """
        if redis.call("get",KEYS[1]) == ARGV[1] then
            return redis.call("del",KEYS[1])
        else
            return 0
        end
        """
        lockname = f'lock:{self.lock_name}'
        unlock = conn.register_script(unlock_script)
        result = unlock(keys=[lockname], args=[identifier])
        if result:
            return True
        else:
            return False

    def sadd(self, setName, *values):
        client = self.getClient()
        return client.sadd(setName, *values)


    def sismember(self, setName, value):
        client = self.getClient()
        return client.sismember(setName, value)

    def spop(self, setName):
        client = self.getClient()
        identifier = self.acquire_lock_with_timeout(client)
        value = 0
        if identifier:
            value = client.spop(setName)
            if not value:
                value = 1
            self.release_lock(client, identifier)

        return value
















# -*- coding: utf-8 -*-
# __author__ : Ricky
# __createTime__ : 2022/7/7 17:53
# __fileName__ : XueshuSpider console_scripts.py
# __devIDE__ : PyCharm
import fire
from CrawlSpider.Utils.SpiderRequest import spiderRequest
from CrawlSpider.Utils.SplashHandler import webSplash
from CrawlSpider.spiders.BaseNewsSpider import BaseNewsSpider
try:
    from CrawlSpider.Utils.twineUtils import build as projectBuild
except:
    pass

class Command:

    @staticmethod
    async def get(*args, url="https://httpbin.org/get", **kwargs):
        """
        isUsingProxy 是否使用代理
        proxy        使用哪个代理
        isSplash     是否动态渲染，这里使用的是 selenium
        form         数据返回的形式，json(字典), text(文本), content(二进制流), origin(即表示原请求对象)
        """
        chromedriverPath = 'chromedriverPath' in kwargs and kwargs.pop('chromedriverPath')
        version = 'version' in kwargs and kwargs.pop('version')
        webSplash.initDriver(chromedriverPath=chromedriverPath, version=version, **kwargs)
        res = await spiderRequest.get(
            url,
            *args,
            **kwargs
        )
        print(f"res:{res}")

    @staticmethod
    async def post(*args, url="https://httpbin.org/post", **kwargs):
        """
       isUsingProxy 是否使用代理
       proxy        使用哪个代理
       form         数据返回的形式，json(字典), text(文本), content(二进制流), origin(即表示原请求对象)
       """

        res = await spiderRequest.get(
            url,
            *args,
            **kwargs
        )
        print(f"res:{res}")

    @staticmethod
    async def spider(*args, url="https://httpbin.org/get",**kwargs):
        """
       isUsingProxy 是否使用代理
       proxy        使用哪个代理
       form         数据返回的形式，json(字典), text(文本), content(二进制流), origin(即表示原请求对象)
       """
        chromedriverPath = 'chromedriverPath' in kwargs and kwargs.pop('chromedriverPath')
        version = 'version' in kwargs and kwargs.pop('version')
        method = 'method' in kwargs and kwargs.pop("method") or 'getList'
        webSplash.initDriver(chromedriverPath=chromedriverPath, version=version, **kwargs)

        spider = BaseNewsSpider(*args, seed=url, *kwargs)
        res = await getattr(spider, method)(**kwargs)
        print(f"res:{res}")
    @staticmethod
    async def build(*args, **kwargs):
        """
        打包项目,基于twine库的使用
        is_publish 是否发布，默认false只打包不上传到pypi
        version    版本号，不填默认为原项目下__version__.py文件中的__version__
        distDir    构建包所在目录，不填则为运行此命令的项目根路径下的dist目录
        **kwargs   即为 twine 所需参数，比如 username, password等
        """
        projectBuild(*args, **kwargs)



def main():
    fire.Fire({
        'build': Command.build,
        'get': Command.get,
        'post': Command.post,
        'spider': Command.spider
    })



# -*- coding: utf-8 -*-
# __author__ : Ricky
# __createTime__ : 2022/4/23 0:08
# __fileName__ : wx_kam_robot mysqlDbSync.py
# __devIDE__ : PyCharm

import pymysql
from dbutils.pooled_db import PooledDB
from contextlib import contextmanager
import traceback
divLen = 5

class MySQLDb:
    def __init__(self, connect_params):
        self.connect_params:dict = connect_params
        self.instanced()

    def instanced(self):
        # , ping=2
        self.poolDB = PooledDB(pymysql, mincached=5, maxcached=20, maxconnections=20, blocking=True, **self.connect_params)

    def get_conn(self):
        if self.poolDB is None:
            self.instanced()
        while 1:
            try:
                return self.poolDB.connection()
            except Exception:
                print(traceback)

    @contextmanager
    def selectAll(self, sql):
        conn_python = self.get_conn()
        cursor_python = conn_python.cursor(pymysql.cursors.DictCursor)
        try:
            cursor_python.execute(sql)
            result = cursor_python.fetchall()

            yield result
        except Exception as e:
            print(traceback.format_exc())
            yield None
        finally:
            cursor_python.close()
            conn_python.close()

    @contextmanager
    def getCount(self, sql):
        if ('count' not in sql.lower()):
            yield 'NoCount'
        else:
            conn_python = self.get_conn()
            cursor_python = conn_python.cursor(pymysql.cursors.DictCursor)
            # sql = 'INSERT INTO {table}({keys}) VALUES ({values})'.format(table=table, keys=keys, values=values)
            try:
                cursor_python.execute(sql)
                result = cursor_python.fetchall()
                # print(result[0])
                yield result
            except Exception as e:
                print(traceback.format_exc())
                yield None
            finally:
                cursor_python.close()
                conn_python.close()

    @contextmanager
    def insertMany(self, sql, params):
        conn_python = self.get_conn()
        cursor_python = conn_python.cursor()
        yield sql
        try:
            cursor_python.executemany(sql, params)
            conn_python.commit()
        except Exception as e:
            print(f'总共插入: {len(params)}条数据')
            conn_python.rollback()
            print(traceback.format_exc())
            write_flag = 0
            length = len(params)

            for i in range(0, length, divLen):
                res = params[i:(i + divLen)]
                try:
                    cursor_python.executemany(sql, res)
                    conn_python.commit()
                except Exception as e:

                    conn_python.rollback()

                    for param in res:
                        try:
                            cursor_python.execute(sql, param)
                            conn_python.commit()
                        except Exception as e:
                            conn_python.rollback()
                            print("param:", param)
                            write_flag += 1
        finally:
            cursor_python.close()
            conn_python.close()

    @contextmanager
    def insertOneByOne(self, sql, params):
        conn_python = self.get_conn()
        cursor_python = conn_python.cursor()
        yield sql
        for param in params:
            try:
                # print(params[0])
                cursor_python.execute(sql, param)
                conn_python.commit()
            except Exception as e:
                print(traceback.format_exc())
                conn_python.rollback()
        else:
            cursor_python.close()
            conn_python.close()

    @contextmanager
    def delete(self, sql, params=None):
        conn_python = self.get_conn()
        cursor_python = conn_python.cursor()
        yield sql
        try:
            if (params):
                cursor_python.execute(sql, params)
            else:
                cursor_python.execute(sql)
            conn_python.commit()
        except Exception as e:
            print(e)
            conn_python.rollback()
        finally:
            cursor_python.close()
            conn_python.close()

    @contextmanager
    def update(self, sql, params=None):
        conn_python = self.get_conn()
        cursor_python = conn_python.cursor()
        yield sql
        try:
            if (params):
                cursor_python.execute(sql, params)
            else:
                cursor_python.execute(sql)
            conn_python.commit()
        except Exception as e:
            print(e)
            conn_python.rollback()
        finally:
            cursor_python.close()
            conn_python.close()

    def table_select(self, table, db=None, condition=None):
        # 'select * from wechat_conf where status=0  and id<10000'
        db = db or self.connect_params.get('db')
        sql = f"select * from {db}.{table} {condition}"
        print(sql)
        with self.selectAll(sql) as rows:
            return rows

    def table_insert(self, table, item, db=None):
        """
        :param item: kwargs={} 字典形式
        :return:
        """
        db = db or self.connect_params.get('db')
        keys = ','.join(item.keys())
        values = ','.join(['%s'] * len(item))
        sql = f'INSERT INTO {db}.{table}({keys}) VALUES ({values})'
        params = [tuple(item.values())]  # 必须是元祖形式
        with self.insertMany(sql, params=params) as fmt_sql:
            print(fmt_sql)
        print('插入完毕: %s' % str(len(params)), end='')

    def table_insertMany(self,table,  itemList:list, db=None):
        """
        :param kwargs: kwargs=[{},...]  嵌套字典形式
        :return:
        """
        db = db or self.connect_params.get('db')
        item = itemList[0]
        keys = ','.join(item.keys())
        values = ','.join(['%s'] * len(item))
        sql = f'INSERT INTO {db}.{table}({keys}) VALUES ({values})'
        params = []
        for item in itemList:
            params.append(tuple(item.values()))  # 子元素必须是元组形式
        with self.insertMany(sql, params=params) as fmt_sql:
            print(fmt_sql)
        print('插入完毕: %s' % str(len(params)), end='')




# -*- coding: utf-8 -*-
# __author__ : Ricky
# __createTime__ : 2022/7/7 15:59
# __fileName__ : XueshuSpider FileUtils.py
# __devIDE__ : PyCharm
from PySide2 import QtCore as qtc


def getDir(fileDirPath, customDir=None):
    fileDir = customDir if customDir else qtc.QDir.home()
    flag = fileDir.cd(fileDirPath)
    if not flag:
        fileDir.mkpath(fileDirPath)
        fileDir.cd(fileDirPath)
    return fileDir
def copyFile(fileName:str, fileDirPath, customDir=None):
    fileDir = getDir(fileDirPath, customDir=customDir)
    newName = fileDir.filePath(fileName.split('/')[-1])
    qtc.QFile.copy(fileName, newName)
    return newName



def mkdir(fileDirPath, customDir=None):
    fileDir = customDir if customDir else qtc.QDir.home()
    flag = fileDir.cd(fileDirPath)
    if not flag:
        while not fileDir.mkpath(fileDirPath):
            qtc.QThread.msleep(200)
            print(f"创建目录: {fileDirPath}")
        fileDir.cd(fileDirPath)
    return fileDir.path()

def delDir(fileDirPath, customDir=None):
    fileDir = customDir if customDir else qtc.QDir.home()
    if fileDir.cd(fileDirPath):
        f = fileDir.removeRecursively()
        print(f"删除目录: {fileDirPath}", f)


def deleteBak(dirPath):
    fileDir = qtc.QDir(dirPath)
    entryInfoList = fileDir.entryInfoList(filters=qtc.QDir.Filter.Files)
    for entryInfo in entryInfoList:
        entryInfo: qtc.QFileInfo

        if entryInfo.suffix() == 'bak':
            fileName = entryInfo.fileName()
            fileDir.remove(fileName)



# -*- coding: utf-8 -*-
# __author__ : Ricky
# __createTime__ : 2022/2/24 下午2:49
# __fileName__ : spiderAssistant AliSDK.py
# __devIDE__ : PyCharm
import uuid
from aliyunsdkcore import client
from aliyunsdkcore.profile import region_provider
import json
from aliyunsdkcore.request import RoaRequest
from aliyunsdkcore.http import format_type
from aliyunsdkgreen.endpoint import endpoint_data


class TextAsyncScanRequest(RoaRequest):

    def __init__(self):
        RoaRequest.__init__(self, 'Green', '2018-05-09', 'TextAsyncScan', 'green')
        self.set_uri_pattern('/green/text/scan')
        self.set_method('POST')
        if hasattr(self, "endpoint_map"):
            setattr(self, "endpoint_map", endpoint_data.getEndpointMap())
        if hasattr(self, "endpoint_regional"):
            setattr(self, "endpoint_regional", endpoint_data.getEndpointRegional())

    def get_ClientInfo(self):
        return self.get_query_params().get('ClientInfo')

    def set_ClientInfo(self, ClientInfo):
        self.add_query_param('ClientInfo', ClientInfo)


class AliClient:
    def __init__(
        self,
        ak=None,
        secret=None,
        region_id="cn-hangzhou",
        **kwargs
    ):
        self.clt = client.AcsClient(ak, secret, region_id, **kwargs)
        self.region_id = region_id


    def TextReview(self, tasks: list, bizType='default', product_name='Green', end_point='green.cn-shanghai.aliyuncs.com'):

        region_provider.modify_point(product_name, self.region_id, end_point=end_point)
        request = TextAsyncScanRequest()
        request.set_accept_format('JSON')
        request.set_headers({
            'Content-Type': format_type.APPLICATION_JSON
        })

        request.set_body_params(
            {
                "scenes": ["antispam"],
                "bizType": bizType,
                "tasks": tasks
            }
        )


        try:
            response = self.clt.do_action_with_exception(request)
            return response
        except Exception as err:
            return err.__str__()





if __name__ == '__main__':
    ali = AliClient()
    response = ali.TextReview([
        {
            'dataId': str(uuid.uuid4()),
            'content': 'marketing strategy and planning in the uk pharmaceutical industry some preliminary findings стратегия маркетинга и планирования в британской фармацевтической промышленности некоторые предварительные результаты'
        }

    ]).decode()

    response = json.loads(response)
    response['data'][0].pop('content')
    print(response)



# -*- coding: utf-8 -*-
# __author__ : Ricky
# __createTime__ : 2022/5/21 23:49
# __fileName__ : XueshuSpider process_monitor.py
# __devIDE__ : PyCharm

import time
import psutil
import fire
import asyncio

class Monitor:
    interval = 60  # 获取CPU，内存使用情况轮询时间间隔
    num = 100
    processMetrics = [None]*num

    @staticmethod
    async def process_monitor(pid=0, pName=None, **kwargs):

        interval = kwargs.get('interval') or Monitor.interval

        pidList = []
        if pid:
            pidList = [pid]
        elif pName is not None:
            pidList = [p.pid for p in psutil.process_iter() if pName in p.name()]

        while 1:
            memTotal = 0.0
            cpuPercent = 0.0
            memPercent = 0.0
            current_time = time.strftime('%Y%m%d-%H%M%S', time.localtime(time.time()))
            for pId in pidList:
                p = psutil.Process(pId)
                cpu_percent = p.cpu_percent()  # better set interval second to calculate like:  p.cpu_percent(interval=0.5)
                cpuPercent += cpu_percent
                mem_percent = p.memory_percent()
                memPercent += mem_percent
                mem_MB = round(p.memory_info().rss / 1024 / 1024, 4)
                memTotal += mem_MB
            line = f"current_time:{current_time}--cpu_percent:{cpuPercent}--mem_percent:{memPercent}--memTotal:{memTotal}"
            print(line)
            await asyncio.sleep(interval)




def run(pid=0, pName=None, method='process_monitor',  **kwargs):

    loop = asyncio.get_event_loop()
    loop.run_until_complete(getattr(Monitor, method)(pid=pid, pName=pName, **kwargs))


if __name__ == '__main__':
    fire.Fire(run)


import datetime
import time
import hashlib
import os
import tempfile
import requests
import json
import zipfile
from urllib.parse import urlparse
from CrawlSpider.conf.settings import headers, uaDic
from PySide2 import QtCore as qtc
import yaml
from CrawlSpider.Utils.Settings import Settings


md5Crypt = qtc.QCryptographicHash(qtc.QCryptographicHash.Md5)

base = [str(x) for x in range(10)] + [ chr(x) for x in range(ord('A'),ord('A')+6)]
def bin2dec(string_num):
    return str(int(string_num, 2))
def dec2hex(string_num):
    num = int(string_num)
    mid = []
    while True:
        if num == 0: break
        num,rem = divmod(num, 16)
        mid.append(base[rem])
    return ''.join([str(x) for x in mid[::-1]])
def bin2hex(string_num):
    return dec2hex(bin2dec(string_num))


def getRealUrl(detailUrl:str, listUrl:str):
    realUrl = detailUrl
    res = urlparse(detailUrl)
    if not res.netloc:
        if detailUrl.startswith('/'):
            result = urlparse(listUrl)
            realUrl = f"{result.scheme}://{result.netloc}{detailUrl}"
        elif detailUrl.startswith('.'):
            suffix = listUrl.split('/')[-1]
            prefix = listUrl.rstrip(suffix)
            realUrl = prefix + detailUrl
    if not res.scheme and detailUrl.startswith("//"):
        scheme = urlparse(listUrl).scheme
        realUrl = f'{scheme}:{detailUrl}'
    if not urlparse(realUrl).netloc:
        realUrl = getRealUrl('/' + realUrl, listUrl)
    return realUrl



def downloadFileFromInet(downloadedUrl, outputDir, outFile):
    resp = requests.get(downloadedUrl, headers = headers)
    status_code = 0
    retry = 3
    output = os.path.join(outputDir, outFile)
    while status_code != 200 and retry:
        status_code = resp.status_code
        if resp.status_code == 200:
            with open(output, 'wb') as fw:
                fw.write(resp.content)
            targetPath = os.path.join(outputDir, unzip(output, outputDir))
            if os.path.splitext(output)[-1] != os.path.splitext(targetPath)[-1]:
                os.remove(output)
            return targetPath
        else:
            print(f"下载 {downloadedUrl} 失败")
        retry -= 1


def md5_dict(item:dict, fields=['snapshotUrl', 'publish_time','title', 'source','author']):
    md5Crypt.reset()
    for field in fields:
        value = item[field]
        if value:
            md5Crypt.addData(value.encode())
    hash_string = bytes(md5Crypt.result().toHex()).decode('UTF-8')
    return hash_string

def md5_text(text:str,salt="12345"):
    md5 = hashlib.md5(salt.encode())
    md5.update(text.encode())
    return md5.hexdigest()

def initDateTime(period=None, days=1):
    tds = []

    if period:
        try:
            start_time = datetime.datetime.strptime(period[0], '%Y-%m-%d')
            end_time = datetime.datetime.strptime(period[1], '%Y-%m-%d')
            for day in range((end_time - start_time).days + 1):
                td = (start_time + datetime.timedelta(days=day)).strftime('%Y-%m-%d')
                ttd = (start_time + datetime.timedelta(days=day + 1)).strftime('%Y-%m-%d')
                print(td, ttd)
                tds.append(td)
        except Exception as e:
            print(e)

    elif days:
        if isinstance(days, int):
            yd = (datetime.datetime.now() - datetime.timedelta(days=days)).strftime('%Y-%m-%d')
            td = (datetime.datetime.now() - datetime.timedelta(days=days - 1)).strftime('%Y-%m-%d')
            print(yd, td)
            tds.append(yd)

    else:
        print('时间参数设置错误')

    return tds





def getOrigin():
    url = 'http://httpbin.org/get'
    res = requests.get(url).json()
    print(res)
    return res.get('origin')

def initUaCache(uaFile=None, version='0.1.11', **kwargs):
    uaFile = uaFile or ''
    DB = os.path.join(
        tempfile.gettempdir(),
        'fake_useragent_{version}.json'.format(
            version=version,
        )
    )
    print(DB)
    if not os.path.exists(DB):
        if uaFile and os.path.isfile(uaFile) and os.path.exists(uaFile):
            with open(uaFile, 'rb') as fr, open(DB, 'wb') as fw:
                fw.write(fr.read())
        elif kwargs.get('url'):
            url = kwargs['url']
            resp = requests.get(url)
            if resp.status_code == 200:
                with open(uaFile, 'rb') as fr, open(DB, 'wb') as fw:
                    fw.write(resp.content)
        elif uaDic.get(version):
            uaData = uaDic[version]
            with open(DB, 'w', encoding='utf-8') as fw:
                fw.write(json.dumps(uaData, ensure_ascii=False, indent=4))




def getWhiteList(whiteIpListUrl):
    if whiteIpListUrl:
        try:
            res = requests.get(whiteIpListUrl).json()
            lists = res['data']['lists']
            return list([item.get('mark_ip') for item in lists])
        except:
            pass
    return []

def initWhite(ip=None, whiteIpListUrl=None, whiteIpSaveUrl=None):
    origin = ip or getOrigin()
    whiteList = getWhiteList(whiteIpListUrl)
    if origin not in whiteList and whiteIpSaveUrl:
        while True:
            url = whiteIpSaveUrl + origin
            res = requests.get(url).json()
            print(res)
            msg = res.get('msg')
            if '再次请求' not in msg:
                break
            time.sleep(3)

def unzip(zipFilePath, output, kw='chromedriver'):
    b = zipfile.is_zipfile(zipFilePath)
    targetPath = zipFilePath
    if b:
        fz = zipfile.ZipFile(zipFilePath, 'r')
        for file in fz.namelist():
            if kw in fz.getinfo(file).filename:
                targetPath = fz.extract(file, output)

    return targetPath


def getDunTextReviewLabels(fileName='../File/dun_text_review_labels.txt'):
    dun_text_review_labels = {}
    with open(fileName, 'r', encoding='utf-8') as fr:
        data = fr.read()
        for line in data.split('\n'):
            if line.strip():
                label, subLabel, desc = line.split('\t')
                dun_text_review_labels.setdefault(label, {})
                dun_text_review_labels[label][subLabel] = desc

    return dun_text_review_labels

class MyEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, bytes):
            return obj.decode('utf-8', 'ignore')
        if isinstance(obj, datetime.datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, datetime.date):
            return obj.strftime('%Y-%m-%d')
        else:
            return json.JSONEncoder.default(self, obj)

def check_contain_chinese(check_str):
    for ch in check_str:
        if u'\u4e00' <= ch <= u'\u9fff':
            return True
        else:
            return False


def get_valueFmt(keys, digital=0):
    temp = []
    # :1,: 2,:3,: 4,:5,: 6,:7,: 8,:9,: 10,:11,: 12
    for i in range(len(keys)):
        if digital:
            temp.append(':' + str(i + 1))
        else:
            temp.append(':' + str(keys[i]))
    return ','.join(temp)


def parseYml(ymlFile):
    with open(ymlFile, 'r') as stream:
        try:
            return yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            raise yaml.YAMLError(exc)


def initConf(configFile):
    if isinstance(configFile, str) and os.path.exists(configFile):
        if configFile.endswith('ini'):
            return Settings(configFile)
        elif configFile.endswith('yaml') or configFile.endswith('yml'):
            return parseYml(configFile)
    else:
        raise ValueError(f"{configFile}: not File")





import asyncio
import copy
import json
import datetime
import traceback
import requests as sync_requests
from aiohttp_requests import requests, Requests
from aiohttp import ClientResponse
from urllib.parse import urlparse
from CrawlSpider.Utils.SplashHandler import webSplash

requests:Requests

headers = {
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.51 Safari/537.36',
    'cookie': 'sessionid=7q8zzketx8qli1siip5btykqlgueuwl0; _gcl_au=1.1.436350073.1646415814; _ga=GA1.2.1481888755.1646415816; _gid=GA1.2.1376185134.1646415816; _fbp=fb.1.1646415815733.1640460909; csrftoken=eujMQHFbEd4sPTgi16qgQ9QGuYzVEKWNIM5tHuCdW0trb1qvhsMXpupQUA8Y1afI; _hjSessionUser_1718031=eyJpZCI6IjA1MjY2MjY3LWYxOTEtNWVlOC04NDU4LTRhZTg3OWJhZWU3OCIsImNyZWF0ZWQiOjE2NDY0MTU4MTU3MTIsImV4aXN0aW5nIjp0cnVlfQ==; ss_cvr=0a7130a3-239f-4113-bd5f-cf429d2829ba|1646415814518|1646415814518|1646428063080|2; crumb=BRXeiGgV13WXNmM2YWQwNmMxZGJmMTAwN2QzZjQxMjFkY2Y5ODc4; _sp_ses.eeee=*; _hjSession_1718031=eyJpZCI6IjRjOThhZDJkLWJiYjUtNGRlYi05N2Y1LWM4YWJhODMwZThiMyIsImNyZWF0ZWQiOjE2NDY1MTk1OTY5MjIsImluU2FtcGxlIjpmYWxzZX0=; _hjAbsoluteSessionInProgress=0; _sp_id.eeee=d33234ab-75c5-4e68-846d-8cbc9b72e965.1646415816.7.1646520509.1646514056.ef959c9c-a67e-4564-80ed-114737664d84'
}



class MyEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, bytes):
            return obj.decode('utf-8', 'ignore')
        if isinstance(obj, datetime.datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, datetime.date):
            return obj.strftime('%Y-%m-%d')
        else:
            return json.JSONEncoder.default(self, obj)



class SpiderRequest:
    async def requestAsync(self,method, url, form="content", **kwargs):
        _headers: dict = (kwargs.get('headers') and kwargs.pop('headers')) or {}
        _headers.update(headers)
        # request: Requests = Requests()
        kwargs.get('chrome_options') and kwargs.pop('chrome_options')
        r = None
        retry = 'retry' in kwargs and kwargs.pop('retry') or 3
        status_code = 200
        closeFlag = 1
        while retry:
            try:
                # resp: ClientResponse = await request.session.get(url, headers=_headers, **kwargs)
                resp: ClientResponse = await requests.session.request(method, url, headers=_headers, **kwargs)
                status_code = resp.status
                if resp and resp.ok:
                    if form == 'json':
                        r = await resp.json()
                    elif form == 'text':
                        r = await resp.text()
                    elif form == 'content':
                        r = await resp.read()
                    elif form == 'origin':
                        r = resp
                        closeFlag = 0
                    if closeFlag:
                        resp.close()
                    break
                elif 400<status_code<500:
                    r = resp
                    resp.close()
                    break
                else:
                    retry -= 1
                    r = await resp.json()
            except Exception as e:
                print("url请求报错:", url)
                print(e)
                retry -= 1
                status_code = 500


        return {
            'status_code': status_code,
            'content': r
        }

    async def requestSync(self,method, url, form="content", **kwargs):
        _headers: dict = (kwargs.get('headers') and kwargs.pop('headers')) or {}
        _headers.update(headers)
        kwargs.get('chrome_options') and kwargs.pop('chrome_options')
        proxyUrl = (kwargs.get('proxy') and kwargs.pop('proxy')) or ""
        if proxyUrl:
            res = urlparse(proxyUrl)
            proxies = {
                res.scheme: res.netloc
            }
            kwargs['proxies'] = proxies
        resp = sync_requests.request(method, url, headers=_headers, **kwargs)
        r = None
        status_code = resp.status_code
        if resp and resp.ok:  # 注意这里是status，不是status_code
            if form == 'json':
                r = resp.json()
            elif form == 'text':
                r = resp.text
            elif form == 'content':
                r = resp.content
            elif form == 'origin':
                r = resp
        return {
            'status_code': status_code,
            'content': r
        }


    async def getAsync(self,url, form="content", **kwargs):
        _headers: dict = (kwargs.get('headers') and kwargs.pop('headers')) or {}
        _headers.update(headers)
        # request: Requests = Requests()
        kwargs.get('chrome_options') and kwargs.pop('chrome_options')
        r = None
        retry = 3
        status_code = 200
        closeFlag = 1
        while retry:
            try:
                # resp: ClientResponse = await request.session.get(url, headers=_headers, **kwargs)
                resp: ClientResponse = await requests.session.get(url, headers=_headers, **kwargs)
                status_code = resp.status
                if resp and resp.ok:
                    if form == 'json':
                        r = await resp.json()
                    elif form == 'text':
                        r = await resp.text()
                    elif form == 'content':
                        r = await resp.read()
                    elif form == 'origin':
                        r = resp
                        closeFlag = 0
                    if closeFlag:
                        resp.close()
                    break
                else:
                    retry -= 1
            except Exception as e:
                print("url请求报错:", url)
                print(e)
                retry -= 1
                status_code = 500

        return {
            'status_code': status_code,
            'content': r
        }


    async def getSync(self,url, form="content", **kwargs):
        _headers: dict = (kwargs.get('headers') and kwargs.pop('headers')) or {}
        _headers.update(headers)
        kwargs.get('chrome_options') and kwargs.pop('chrome_options')
        proxyUrl = (kwargs.get('proxy') and kwargs.pop('proxy')) or ""
        if proxyUrl:
            res = urlparse(proxyUrl)
            proxies = {
                res.scheme: res.netloc
            }
            kwargs['proxies'] = proxies
        resp = sync_requests.get(url, headers=_headers, **kwargs)
        r = None
        status_code = resp.status_code
        if resp and resp.ok:  # 注意这里是status，不是status_code
            if form == 'json':
                r = resp.json()
            elif form == 'text':
                r = resp.text
            elif form == 'content':
                r = resp.content
            elif form == 'origin':
                r = resp
        return {
            'status_code': status_code,
            'content': r
        }


    async def postAsync(self, url, form="content", **kwargs):
        _headers: dict = (kwargs.get('headers') and kwargs.pop('headers')) or {}
        _headers.update(headers)
        kwargs.get('chrome_options') and kwargs.pop('chrome_options')
        r = None
        retry = 3
        status_code = 200
        closeFlag = 1
        while retry:
            try:
                resp: ClientResponse = await requests.session.post(url, headers=_headers, **kwargs)
                status_code = resp.status
                if resp and resp.ok:  # 注意这里是status，不是status_code
                    if form == 'json':
                        r = await resp.json()
                    elif form == 'text':
                        r = await resp.text()
                    elif form == 'content':
                        r = await resp.read()
                    elif form == 'origin':
                        r = resp
                        closeFlag = 0
                    if closeFlag:
                        resp.close()

                    break
                else:
                    retry -= 1
            except Exception as e:
                print("url请求报错:", url)
                print(e)
                retry -= 1
                status_code = 500
        return {
            'status_code': status_code,
            'content': r
        }

    async def postSync(self, url, form="content", **kwargs):
        _headers: dict = (kwargs.get('headers') and kwargs.pop('headers')) or {}
        _headers.update(headers)
        kwargs.get('chrome_options') and kwargs.pop('chrome_options')
        proxyUrl = (kwargs.get('proxy') and kwargs.pop('proxy')) or ""
        if proxyUrl:
            res = urlparse(proxyUrl)
            proxies = {
                res.scheme: res.netloc
            }
            kwargs['proxies'] = proxies
        resp =  sync_requests.post(url, headers=_headers, **kwargs)
        r = None
        status_code = resp.status_code

        if resp.ok:  # 注意这里是status，不是status_code
            if form == 'json':
                r = resp.json()
            elif form == 'text':
                r = resp.text
            elif form == 'content':
                r = resp.content
            elif form == 'origin':
                r = resp

        return {
            'status_code': status_code,
            'content': r
        }

    async def get(self, url, isAllow302=True, isUsingProxy=False, isSplash=False, **kwargs):
        if isUsingProxy:
            ret = await self.proxy(self.get_, url, isAllow302=isAllow302, isSplash=isSplash, **kwargs)
        else:
            ret = await self.get_(url, isAllow302=isAllow302, isSplash=isSplash,**kwargs)
        return ret

    async def get_(self, url, isAllow302=True,  isSplash=False,isAsync=True, **kwargs):
        if isSplash:
            return await self.splash(url, **kwargs)
        elif isAsync:
            ret = await self.getAsync(url, **kwargs)
            return ret
        elif not isAsync:
            return await self.getSync(url, **kwargs)

    @classmethod
    async def splash(cls,url, **kwargs):
        loop = asyncio.get_running_loop()
        # --proxy-server=socks5://xxx.xxx:xxx
        options = kwargs.get('chrome_options')
        splashTime = kwargs.get('splashTime') or 0.2
        isUsingNewDriver = kwargs.get('isUsingNewDriver')
        driver = None
        if isUsingNewDriver:
            driver = webSplash.getNewDriver(options)
        elif options:
            driver = webSplash.getDriver(options=options)
        future = loop.run_in_executor(None, webSplash.getSource, url, driver, splashTime)
        res = await future
        return res

    @classmethod
    def splashSync(cls, url, **kwargs):
        options = kwargs.get('chrome_options')
        splashTime = kwargs.get('splashTime') or 0.2
        isUsingNewDriver = kwargs.get('isUsingNewDriver')
        driver = None
        if isUsingNewDriver:
            driver = webSplash.getNewDriver(options)
        elif options:
            driver = webSplash.getDriver(options=options)
        res = webSplash.getSource(url, driver, splashTime)
        return res


    @classmethod
    async def proxy(cls,fn, url, isAllow302=True,  isSplash=False, **kwargs):
        from CrawlSpider.Utils.RetrieveProxy import RetrieveProxy
        _kwargs = copy.deepcopy(kwargs)
        isFree = True
        proxyType = _kwargs.pop('proxyType') if 'proxyType' in _kwargs else 'abuyun'
        protocol = _kwargs.pop('protocol') if 'protocol' in _kwargs else 'http'
        if 'isFree' in _kwargs:
            isFree = _kwargs.pop('isFree')
        if isFree:
            retry = 3
            proxy, _ = await RetrieveProxy.getProxyFree()
            print("获取到的免费代理IP:", proxy)
            # proxies requests {}
            httpProxy = f"{protocol}://{proxy}"
            _kwargs.update({
                'proxy': httpProxy,
                'chrome_options': [f'--proxy-server={httpProxy}']
            })
            while retry:
                try:
                    ret = await fn(url, isAllow302=isAllow302, isSplash=isSplash, **_kwargs)
                    return ret
                except Exception as e:
                    print("e", e)
                    retry -= 1
            ret = await RetrieveProxy.delete(proxy)
            print("删除结果:", ret)
        proxy, _ = await RetrieveProxy.getProxy(proxyType=proxyType)

        if proxy:

            # ret = await RetrieveProxy.addProxy({
            #     'source': "proxyType",
            #     'proxy': proxy
            # })
            # print("添加结果:", ret)
            httpProxy = f"{protocol}://{proxy}" if not proxy.startswith(protocol) else proxy

            _kwargs.update({
                'proxy': httpProxy,
                'chrome_options': [f'--proxy-server={httpProxy}']
            })
            try:
                ret = await fn(url, isAllow302=isAllow302, isSplash=isSplash, **_kwargs)
                return ret
            except Exception as e:
                print("e", e)
                return None


    async def post(self, url, isAllow302=True, isUsingProxy=False, isSplash=False,**kwargs):
        if isUsingProxy:
            ret = await self.proxy(self.post_, url, isAllow302=isAllow302, isSplash=isSplash, **kwargs)
        else:
            ret = await self.post_(url, isAllow302=isAllow302, isSplash=isSplash, **kwargs)

        return ret

    async def post_(self, url, isAllow302=True, isSplash=False,isAsync=True,**kwargs):
        if isAsync:
            ret = await self.postAsync(url,  **kwargs)
        else:
            ret = await self.postSync(url,  **kwargs)
        return ret
    @classmethod
    def toJson(cls, data,**kwargs):
        return json.dumps(data, ensure_ascii=False, cls=MyEncoder, **kwargs)
    @classmethod
    def fromJson(cls, data, encoding='utf-8', **kwargs):
        return json.loads(data, encoding=encoding, **kwargs)


spiderRequest = SpiderRequest()



# -*- coding: utf-8 -*-
# __author__ : Ricky
# __createTime__ : 2022/4/18 2:55
# __fileName__ : XueshuSpider DunSDK.py
# __devIDE__ : PyCharm
import hashlib
import time
import random
import requests
import json


class TextCheckAPI(object):
    """文本批量在线检测接口示例代码"""

    API_URL = "http://as.dun.163.com/v5/text/batch-check"
    VERSION = "v5.2"

    def __init__(self, secret_id, secret_key, business_id):
        """
        Args:
            secret_id (str) 产品密钥ID，产品标识
            secret_key (str) 产品私有密钥，服务端生成签名信息使用
            business_id (str) 业务ID，易盾根据产品业务特点分配
        """
        self.secret_id = secret_id
        self.secret_key = secret_key
        self.business_id = business_id



    def gen_signature(self, params=None):
        """生成签名信息
           Args:
               secretKey 产品私钥
               params 接口请求参数，不包括signature参数
       """
        params_str = ""
        for k in sorted(params.keys()):
            params_str += str(k) + str(params[k])
        params_str += self.secret_key
        return hashlib.md5(params_str.encode()).hexdigest()

    def check(self, params):
        """请求易盾接口
        Args:
            params (object) 请求参数
        Returns:
            请求结果，json格式
        """
        params["secretId"] = self.secret_id
        params["businessId"] = self.business_id
        params["version"] = self.VERSION
        params["timestamp"] = int(time.time() * 1000)
        params["nonce"] = int(random.random() * 100000000)
        # params["signatureMethod"] = "SM3"  # 签名方法，默认MD5，支持SM3
        params["signature"] = self.gen_signature(params)

        try:
            res = requests.post(self.API_URL, data=params).json()
            return res
        except Exception as ex:
            print("调用API接口失败:", str(ex))


if __name__ == "__main__":
    """示例代码入口"""
    businessId = '6ad86b847de443ce36c7fae4854e61a0'
    secretId = '1389d67989cbdd09acd236d173dda841'
    secretKey = '3b8e223c4b27d021b85174a3d2a6fd0e'
    api = TextCheckAPI(secretId, secretKey, businessId)

    # 私有请求参数
    texts: list = []
    text1 = {
        "dataId": "ebfcad1c-dba1-490c-b4de-e784c2691768",
        "content": "易盾批量检测接口！v5接口!",
    }
    text2 = {
        "dataId": "ebfcad1c-dba1-490c-b4de-e784c2691678",
        "content": "易盾批量检测接口！v5接口!",
    }
    texts.append(text1)
    texts.append(text2)
    params = {
        "texts": json.dumps(texts),
        # "checkLabels": "200, 500"  # 指定过检分类
    }

    ret = api.check(params)

    code: int = ret["code"]
    msg: str = ret["msg"]
    if code == 200:
        resultArray: list = ret["result"]
        if resultArray is not None and len(resultArray) > 0:
            for result in resultArray:
                dataId: str = result["dataId"]
                taskId: str = result["taskId"]
                action: int = result["action"]
                status: int = result["status"]
                print("dataId=%s，批量文本提交返回taskId:%s" % (dataId, taskId))
                if status == 0:
                    labelArray: list = result["labels"]
                    for labelItem in labelArray:
                        label: int = labelItem["label"]
                        level: int = labelItem["level"]
                        details: dict = labelItem["details"]
                        hintArray: list = details["hint"]
                        subLabels: list = labelItem["subLabels"]
                    if action == 0:
                        print("taskId: %s, 文本机器检测结果: 通过" % taskId)
                    elif action == 1:
                        print("taskId: %s, 文本机器检测结果: 嫌疑, 需人工复审, 分类信息如下: %s" % (taskId, labelArray))
                    elif action == 2:
                        print("taskId=%s, 文本机器检测结果: 不通过, 分类信息如下: %s" % (taskId, labelArray))
                else:
                    print("提交失败")
    else:
        print("ERROR: code=%s, msg=%s" % (ret["code"], ret["msg"]))


import os.path

import boto3
from botocore import paginate

ks = {
    # 'service_name',
    # 'region_name',
    'api_version',
    'use_ssl',
    'verify',
    'endpoint_url',
    'aws_access_key_id',
    'aws_secret_access_key',
    'aws_session_token',
    'config',
}

class MyFile:
    content = None
    def write(self, data: bytes):
        self.content = data
    def read(self, n=0):
        if not n:
            return self.content
        else:
            return self.content[:n]
    def close(self):
        self.content = None

class AwsSDK:

    def __init__(self, service_name='s3', region_name='cn-north-1', bucket_name='', **kwargs):
        kws = {k:kwargs.get(k) for k in ks}

        self._service_name = service_name
        self.s3 = boto3.resource(service_name, region_name, **kws)
        self.s3Client = boto3.client(service_name, region_name, **kws)
        self.bucket_name = bucket_name



    @property
    def service_name(self):
        return self._service_name

    def getBuckets(self):

        return [bucket.name for bucket in self.s3.buckets.all()]

    def getBucket(self, bucket_name):
        return self.s3.Bucket(bucket_name or self.bucket_name)

    def uploadFile(self, fileKey, fileName=None, stream=None, bucket_name=None):
        # Upload a new file
        if not any([fileName, stream]):
            raise ValueError(f"fileName, stream 不能同时为空")
        if not stream:
            stream = open(fileName, 'rb')
        if not hasattr(stream, 'read'):
            raise AttributeError(f"stream hasn't read attribute")
        bucket = self.getBucket(bucket_name)
        res = bucket.upload_fileobj(stream, Key=fileKey)

        stream.close()
        return res




    def getDirs(self, bucket_name=None):
        paginator = self.s3Client.get_paginator('list_objects')
        result: paginate.PageIterator = paginator.paginate(Bucket=bucket_name or self.bucket_name, Delimiter='/')
        return [prefix.get('Prefix') for prefix in result.search('CommonPrefixes')]


    def getObjContent(self, fileKey, fileName=None, bucket_name=''):
        file = MyFile()
        content = b''
        bucket = self.getBucket(bucket_name)
        if fileName:
            ret = bucket.download_file(fileKey, fileName)
        else:
            ret = bucket.download_fileobj(fileKey, file)
            content = file.content
        return ret, content










from CrawlSpider.Utils.SpiderRequest import spiderRequest
from bs4 import BeautifulSoup,Tag
def getPlainContent(tag):
    return tag.has_attr('id') and tag['id'] == 'js_content'
def getVideoContent(tag):
    return tag.has_attr('id') and tag['id'] == 'js_content'
class WxGzh:
    @classmethod
    async def plainText(cls, soup:Tag):
        contents = soup.find_all(getPlainContent)
        return contents

    @classmethod
    async def videoText(cls, soup:Tag):
        contents = soup.find_all(getVideoContent)
        return contents



async def test(url):
    res = await spiderRequest.get(url)
    soup = BeautifulSoup(res.decode(), 'lxml')
    contents = await WxGzh.videoText(soup)
    print(contents)






if __name__ == '__main__':
    import asyncio
    loop = asyncio.get_event_loop()
    url = "https://mp.weixin.qq.com/s?__biz=MzA5ODgxNDAyMQ==&mid=2654690723&idx=2&sn=3451b209eab2200c534f37fe32820895&chksm=8b442c89bc33a59fd0c58201a9114e7e282d5a08ecbf12d06e64780833b2058ee44de0c214f7&&&sessionid=0&scene=126&clicktime=1627463401&enterid=1627463401"
    s = loop.create_task(test(url))
    loop.run_until_complete(s)










from CrawlSpider.Utils.ResHandler import ImgSrcReplace

class BaseETL:

    @classmethod
    async def bodyTransfer(cls, data):
        """
        测试body的图片和链接的置换
        """
        body = data.get('new_content')
        imgHint = "未做转换"
        if body:
            imgFileScope = data.get('imgFileScope')
            if not imgFileScope:
                url = data.get('url')
                print(url)
                if url:
                    imgFileScope = url.rstrip(url.split('/')[-1])
            body, imgHint = await ImgSrcReplace.common(body, imgFileScope=imgFileScope)
        return {
            'status': 0,
            'new_content': body,
            'imgHint': imgHint
        }

    @classmethod
    async def delTagsExceptImg(cls, htmlStr: str = None, soup=None, imgAttr='src'):
        # 这里不需要，因为已经做过前置处理了 preProcess
        # htmlStr = cls.cleanWebChars(htmlStr)
        res = await ImgSrcReplace.delTagsExceptImg(htmlStr, soup, imgAttr)
        return res

    @classmethod
    async def cleanWebChars(cls, htmlStr):
        charList = ['&nbsp;', '&amp;', '&lt;', '&gt;', '&quot;', '&qpos;']
        for charStr in charList:
            if charStr == '&amp;':
                htmlStr = htmlStr.replace(charStr, '&')
            else:
                htmlStr = htmlStr.replace(charStr, '')

        return htmlStr



baseETL = BaseETL()










from io import RawIOBase

class MyFileStream(RawIOBase):

    def __init__(self, content:bytes):
        self.content = content


    def read(self, n:int=-1):  # real signature unknown
        return self.content[:n]

    def readall(self, *args, **kwargs):  # real signature unknown
        """ Read until EOF, using multiple read() call. """
        return self.content







# -*- coding: utf-8 -*-
# __author__ : Ricky
# __createTime__ : 2021/1/21 18:15
# __fileName__ : GoldenCoordinateV2 setup.py.py
# __devIDE__ : PyCharm

from setuptools import setup, find_packages
import os
import sys



requires = [
    'pyperclip==1.8.2',
    'PySide2==5.15.2',
    'PySocks==1.7.1',
    'PyYAML==5.4.1',
    'PyExecJS==1.5.1',
    'protobuf==3.14.0',
    'pkginfo==1.8.3',
    'cos-python-sdk-v5==1.9.15',
    'requests==2.26.0',
    'beautifulsoup4==4.10.0',
    'lxml==4.6.3',
    'DBUtils==3.0.2',
    'PyMySQL==0.9.3',
    'selenium==4.0.0',
    'aiofiles==0.8.0',
    'aiohttp==3.8.1',
    'aiohttp-requests==0.1.3',
    'fake-useragent==0.1.11',
    'retrying==1.3.3',
    'redis==4.3.3',
    'aiomysql==0.0.22',
    'fire==0.4.0',
    'psutil==5.9.1',
    'urllib3==1.26.6',
    'boto3==1.24.24',
    'botocore==1.27.24',
    'aliyun-python-sdk-core-v3==2.13.33',
    'aliyun-python-sdk-green==3.6.5',
    'aliyun-python-sdk-kms==2.15.0',
    'tenacity==8.0.1',
    "psycopg2-binary",
    "psycopg",
    "pytz",
    "pyyaml==5.4.1"
]

# 'setup.py publish' shortcut.
if sys.argv[-1] == "publish":
    os.system("python setup.py sdist bdist_wheel")
    os.system("twine upload dist/*")
    sys.exit()

about = {}
here = os.path.abspath(os.path.dirname(__file__))
with open("__version__.py", "r", encoding="utf-8") as f:
    exec(f.read(), about)

with open("README.md", "r", encoding="utf-8") as f:
    readme = f.read()

setup(

    name=about["__title__"],
    version=about["__version__"],
    description=about["__description__"],
    long_description=readme,
    long_description_content_type="text/markdown",
    author=about["__author__"],
    author_email=about["__author_email__"],
    url=about["__url__"],
    license=about["__license__"],
    keywords='spider scrapy beatifulsoup xpath 正则',
    project_urls={
        'Author Website': 'https://blog.csdn.net/qq_36154755',
    },
    include_package_data=True,
    packages=find_packages(),
    install_requires=requires,
    python_requires='>=3.6',
    extras_require={
        "asyncLoop": [
            'uvloop'
        ],
        "build": [
            "twine"
        ]
    },
    entry_points={
        'console_scripts': [
            'CrawlSpiderUtils = CrawlSpider.Utils.__main__:main'
        ]
    }
)

# python CrawlSpider/Utils/twineUtils.py build --username=None --password=None --version=None --is_publish=False









from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from CrawlSpider.Utils.FileUtils import mkdir
from CrawlSpider.Utils.Helper import downloadFileFromInet
from CrawlSpider.conf.settings import chromePath, chromeDriverDwnUrl
import time
import copy
import os
import platform
sysType = platform.system()


class WebSplash:
    def __init__(self, chromedriverPath=None, **kwargs):
        chrome_options = Options()
        chrome_options.add_argument('--headless')
        chrome_options.add_argument('--no-sandbox')
        chrome_options.add_argument('--disable-gpu')
        chrome_options.add_argument('--disable-dev-shm-usage')

        
        # close chrome
        # chrome_options.add_experimental_option("detach", True)
        ## 不显示"chrome正受到自动测试软件的控制"
        # chrome_options.add_argument('disable-infobars')

        self.chrome_options = chrome_options
        # 无头浏览器需要传入参数在实例化的浏览器对象中*****
        self.driver: webdriver.Chrome = None
        # 让浏览器指定url发起请求 经测试它有动态加载数据
        self.chromedriverPath = chromedriverPath
        self.initDriver(chromedriverPath=chromedriverPath, **kwargs)

    def initDriver(self, chromedriverPath=None, version=None, **kwargs):
        # '//chromedriver_win32.zip'

        if not chromedriverPath:
            version = version or '102.0.5005.61'
            chromeDriverName = 'chromedriver_win32' if sysType == 'Windows' else 'chromedriver_linux64' if sysType == 'Linux' else 'chromedriver_mac64'
            chromeDriverDir = mkdir(f".chrome_driver/{version}")
            pathList = os.listdir(chromeDriverDir)
            if len(pathList) != 1 or pathList[0].endswith('zip'):
                chromedriverPath = downloadFileFromInet(f"{chromeDriverDwnUrl}/{version}/{chromeDriverName}.zip", outputDir=chromeDriverDir, outFile=f"{chromeDriverName}.zip")
                # 给 Linux 上的文件赋予可执行权限
                if sysType == 'Linux' and os.path.exists(chromedriverPath):
                    os.system(f"chmod 777 {chromedriverPath}")
            else:
                chromedriverPath = os.path.join(chromeDriverDir, pathList[0])

        self.chromedriverPath = chromedriverPath

    def getSource(self, url, driver=None, splashTime=0.2):
        # 无头浏览器设置（*********增加爬取效率）
        if not driver:
            driver = self.getNewDriver()

        driver.get(url)
        # sleep 0.2s
        time.sleep(splashTime)
        page_source: str = driver.page_source
        # 关闭当前标签页,一直使用get请求，其实使用的都是main标签页，所以不要在这里关闭，一旦关闭就表示浏览器也被关闭
        # driver.close()
        return {
            'content': page_source.encode(),
            'driver': driver
        }

    def disconnect(self):
        if sysType == 'Windows':
            try:
                import win32process
                print(u"正在关闭Chrome进程")
                self.driver.close()
                self.driver.quit()
                win32process.TerminateProcess(self.chrome[0], 0)
                os.system("taskkill /IM software_reporter_tool.exe /F")
            except Exception as e:
                print(f"e:{e}")


    def getDriver(self, options: list=None):
        if not options:
            if hasattr(self, 'driver') and self.driver:
                return self.driver
            else:
                self.driver = webdriver.Chrome(executable_path=self.chromedriverPath, options=self.chrome_options)
                return self.driver
        else:
            chrome_options = copy.deepcopy(self.chrome_options)
            chrome_options.arguments.extend(options)
            driver = webdriver.Chrome(executable_path=self.chromedriverPath, options=chrome_options)
            return driver


    def getNewDriver(self, options:list=None, isHeadless=True, debugger_address=None):



        chrome_options = copy.deepcopy(self.chrome_options)
        if options:
            chrome_options.arguments.extend(options)
        if '--headless' not in chrome_options.arguments or not isHeadless:
            chrome_options.headless = isHeadless

        if debugger_address:
            chrome_options.debugger_address = debugger_address

        self.driver = webdriver.Chrome(executable_path=self.chromedriverPath, options=chrome_options)
        return self.driver

    def launch_chrome(self, chrome_debugger_port=9224):
        if sysType == 'Windows':
            import win32process
            self.chrome_port = chrome_debugger_port
            if os.path.exists(chromePath):
                command = f"'{chromePath}' --remote-debugging-port={chrome_debugger_port}"
                print(u"如果Chrome白屏，请使用CMD手动运行以下命令:\n{}".format(command))
                self.chrome = win32process.CreateProcess(None, "{} --remote-debugging-port={}".format(chromePath,
                                                                                                      self.chrome_port),
                                                         None, None, 0, 0, None, None, win32process.STARTUPINFO())
            else:
                print(u"未找到Chrome安装目录")
                exit(-1)
            return f"127.0.0.1:{self.chrome_port}"

    def connect(self, options:list=None, page_load_timeout=60, script_timeout=60, chrome_debugger_port=9224, isHeadless=True):
        debugger_address = self.launch_chrome(chrome_debugger_port=chrome_debugger_port)
        driver = self.getNewDriver(options=options, isHeadless=isHeadless, debugger_address=debugger_address)
        driver.set_page_load_timeout(page_load_timeout)
        driver.set_script_timeout(script_timeout)
        return driver

webSplash = WebSplash()





from CrawlSpider.date_infer.date_elements import Filler


class If(object):
    """
    Top-level rule
    """

    def __init__(self, condition, action):
        """
        Initialize the rule with a condition clause and an action clause that will be executed
        if condition clause is true.
        """
        self.condition = condition
        self.action = action

    def execute(self, elem_list):
        """
        If condition, return a new elem_list provided by executing action.
        """
        if self.condition.is_true(elem_list):
            return self.action.act(elem_list)
        else:
            return elem_list


class ConditionClause(object):
    """
    Abstract class for a condition clause
    """

    def is_true(self, elem_list):
        """
        Return true if condition is true for the given input.
        """
        raise NotImplementedError()


class ActionClause(object):
    """
    Abstract class for an action clause
    """

    def act(self, elem_list):
        """
        Return a new instance of elem_list permuted by the action
        """
        raise NotImplementedError()


class And(ConditionClause):
    """
    Returns true if all conditions are true.
    """

    def __init__(self, *clauses):
        self.clauses = clauses

    def is_true(self, elem_list):
        for clause in self.clauses:
            if not clause.is_true(elem_list):
                return False
        return True


class Contains(ConditionClause):
    """
    Returns true if all requirements are found in the input
    """

    def __init__(self, *requirements):
        self.requirements = requirements

    def is_true(self, elem_list):
        for requirement in self.requirements:
            if requirement not in elem_list:
                return False
        return True


class Duplicate(ConditionClause):
    """
    Returns true if there is more than one instance of elem in elem_list.
    """

    def __init__(self, elem):
        self.elem = elem

    def is_true(self, elem_list):
        return elem_list.count(self.elem) > 1


class KeepOriginal(object):
    """
    In sequences, this stands for 'keep the original value'
    """

    pass


class Next(ConditionClause):
    """
    Return true if A and B are found next to each other in the elem_list (with zero or more Filler elements
    between them).
    """

    def __init__(self, a_elem, b_elem):
        self.a_elem = a_elem
        self.b_elem = b_elem

    def is_true(self, elem_list):
        a_positions = []
        b_positions = []
        for index, elem in enumerate(elem_list):
            if elem == self.a_elem:
                a_positions.append(index)
            elif elem == self.b_elem:
                b_positions.append(index)

        for a_position in a_positions:
            for b_position in b_positions:
                left = min(a_position, b_position)
                right = max(a_position, b_position)
                between = elem_list[left + 1 : right - 1]
                if len(between) == 0 or all([type(e) is Filler] for e in between):
                    return True
        return False


class Sequence(ConditionClause):
    """
    Returns true if the given sequence is found in elem_list. The sequence consists of date elements
    and wild cards.

    Wild cards:
    . (period): Any single date element (including Filler)
    """

    def __init__(self, *sequence):
        self.sequence = sequence

    def is_true(self, elem_list):
        seq_pos = (
            0
        )  # if we find every element in sequence (pos == length(self.sequence), then a match is found

        for elem in elem_list:
            if self.match(elem, self.sequence[seq_pos]):
                seq_pos += 1
                if seq_pos == len(self.sequence):
                    return True
            else:
                seq_pos = 0  # reset if we exit sequence
        return False

    @staticmethod
    def match(elem, seq_expr):
        """
        Return True if elem (an element of elem_list) matches seq_expr, an element in self.sequence
        """
        if type(seq_expr) is str:  # wild-card
            if seq_expr == ".":  # match any element
                return True
            elif seq_expr == "\d":
                return elem.is_numerical()
            elif seq_expr == "\D":
                return not elem.is_numerical()
            else:  # invalid wild-card specified
                raise LookupError("{0} is not a valid wild-card".format(seq_expr))
        else:  # date element
            return elem == seq_expr

    @staticmethod
    def find(find_seq, elem_list):
        """
        Return the first position in elem_list where find_seq starts
        """
        seq_pos = 0
        for index, elem in enumerate(elem_list):
            if Sequence.match(elem, find_seq[seq_pos]):
                seq_pos += 1
                if seq_pos == len(find_seq):  # found matching sequence
                    return index - seq_pos + 1
            else:  # exited sequence
                seq_pos = 0
        raise LookupError("Failed to find sequence in elem_list")


class Swap(ActionClause):
    """
    Returns elem_list with one element replaced by another
    """

    def __init__(self, remove_me, insert_me):
        self.remove_me = remove_me
        self.insert_me = insert_me

    def act(self, elem_list):
        copy = elem_list[:]
        pos = copy.index(self.remove_me)
        copy[pos] = self.insert_me
        return copy


class SwapDuplicateWhereSequenceNot(ActionClause):
    """
    Replace remove_me with insert_me in the case where remove_me is not part of the sequence.
    """

    def __init__(self, remove_me, insert_me, seq):
        self.remove_me = remove_me
        self.insert_me = insert_me
        self.seq = seq

    def act(self, elem_list):
        copy = elem_list[:]

        start_pos = Sequence.find(self.seq, copy)
        end_pos = start_pos + len(
            self.seq
        )  # do not replace within [start_pos, end_pos)

        for index, elem in enumerate(copy):
            if start_pos <= index < end_pos:  # within sequence
                continue
            else:  # outside of sequence
                if elem == self.remove_me:
                    copy[index] = self.insert_me
                    return copy

        raise LookupError(
            "Failed to find element {0} to replace with {1} in {2} ignoring {3} between [{4},{5})".format(
                self.remove_me, self.insert_me, copy, self.seq, start_pos, end_pos
            )
        )


class SwapSequence(ActionClause):
    """
    Returns elem_list with sequence replaced with another sequence
    """

    def __init__(self, find_seq, swap_seq):
        self.find_seq = find_seq
        self.swap_seq = swap_seq

    def act(self, elem_list):
        copy = elem_list[:]

        start_pos = Sequence.find(self.find_seq, copy)
        for index, replacement in enumerate(self.swap_seq):
            if replacement is not KeepOriginal:
                copy[start_pos + index] = replacement

        # If we intend to delete items, we put None in the swap_seq and then clean up the list here
        while None in copy:
            copy.remove(None)

        return copy
